@@ -309,7 +309,7 @@ libGAP_Obj libGAP_DoFail0args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 0);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -323,7 +323,7 @@ libGAP_Obj libGAP_DoFail1args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 1);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -338,7 +338,7 @@ libGAP_Obj libGAP_DoFail2args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 2);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -354,7 +354,7 @@ libGAP_Obj libGAP_DoFail3args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 3);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -371,7 +371,7 @@ libGAP_Obj libGAP_DoFail4args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 4);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -389,7 +389,7 @@ libGAP_Obj libGAP_DoFail5args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 5);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -408,7 +408,7 @@ libGAP_Obj libGAP_DoFail6args (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, 6);
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -422,7 +422,7 @@ libGAP_Obj libGAP_DoFailXargs (
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
     argx =libGAP_NargError(self, libGAP_LEN_LIST(args));
-    return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
+    return libGAP_CallFuncList( self, argx );
 }
 
 
@@ -1493,23 +1493,13 @@ libGAP_Obj libGAP_FuncCALL_FUNC (
 */
 libGAP_Obj libGAP_CallFuncListOper;
 
-libGAP_Obj libGAP_FuncCALL_FUNC_LIST (
-    libGAP_Obj                 self,
-    libGAP_Obj                 func,
-    libGAP_Obj                 list )
+libGAP_Obj libGAP_CallFuncList ( libGAP_Obj func, libGAP_Obj list )
 {
     libGAP_Obj                 result;         /* result                          */
     libGAP_Obj                 list2;          /* list of arguments               */
     libGAP_Obj                 arg;            /* one argument                    */
     libGAP_UInt                i;              /* loop variable                   */
-
-    /* check that the second argument is a list                            */
-    while ( ! libGAP_IS_SMALL_LIST( list ) ) {
-        list = libGAP_ErrorReturnObj(
-            "CallFuncList: <list> must be a small list",
-            0L, 0L,
-            "you can replace <list> via 'return <list>;'" );
-    }
+   
 
     if (libGAP_TNUM_OBJ(func) == libGAP_T_FUNCTION) {
 
@@ -1551,12 +1541,27 @@ libGAP_Obj libGAP_FuncCALL_FUNC_LIST (
         result = libGAP_CALL_XARGS( func, list2 );
       }
     } else {
-      result = libGAP_DoOperation2Args(self, func, list);
+      result = libGAP_DoOperation2Args(libGAP_CallFuncListOper, func, list);
     }
     /* return the result                                                   */
     return result;
+
 }
 
+libGAP_Obj libGAP_FuncCALL_FUNC_LIST (
+    libGAP_Obj                 self,
+    libGAP_Obj                 func,
+    libGAP_Obj                 list )
+{
+  /* check that the second argument is a list                            */
+    while ( ! libGAP_IS_SMALL_LIST( list ) ) {
+        list = libGAP_ErrorReturnObj(
+            "CallFuncList: <list> must be a small list",
+            0L, 0L,
+            "you can replace <list> via 'return <list>;'" );
+    }
+    return libGAP_CallFuncList(func, list);
+}
 
 /****************************************************************************
 **
