@@ -119,6 +119,8 @@
 
 #include        "saveload.h"            /* saving and loading              */
 
+#include        "code.h"
+#include        "tls.h"
 
 /****************************************************************************
 **
@@ -144,6 +146,16 @@
 */
 libGAP_Obj libGAP_ResultCyc;
 
+void libGAP_GrowResultCyc(libGAP_UInt size) {
+    libGAP_Obj *res;
+    libGAP_UInt i;
+    if ( libGAP_LEN_PLIST(libGAP_TLS(libGAP_ResultCyc)) < size ) {
+        libGAP_GROW_PLIST( libGAP_TLS(libGAP_ResultCyc), size );
+        libGAP_SET_LEN_PLIST( libGAP_TLS(libGAP_ResultCyc), size );
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
+        for ( i = 0; i < size; i++ ) { res[i] = libGAP_INTOBJ_INT(0); }
+    }
+}
 
 /****************************************************************************
 **
@@ -380,19 +392,19 @@ libGAP_Int             libGAP_LtCycNot (
 **
 *F  ConvertToBase(<n>)  . . . . . . convert a cyclotomic into the base, local
 **
-**  'ConvertToBase'  converts the cyclotomic  'ResultCyc' from the cyclotomic
+**  'ConvertToBase'  converts the cyclotomic  'TLS(ResultCyc)' from the cyclotomic
 **  field  of <n>th roots of  unity, into the base  form.  This means that it
 **  replaces every root $e_n^i$ that does not belong to the  base by a sum of
 **  other roots that do.
 **
-**  Suppose that $c*e_n^i$ appears in 'ResultCyc' but $e_n^i$ does not lie in
+**  Suppose that $c*e_n^i$ appears in 'TLS(ResultCyc)' but $e_n^i$ does not lie in
 **  the base.  This happens  because, for some  prime $p$ dividing $n$,  with
 **  maximal power $q$, $i \in (n/q)*[-(q/p-1)/2..(q/p-1)/2]$ mod $q$.
 **
 **  We take the identity  $1+e_p+e_p^2+..+e_p^{p-1}=0$, write it  using $n$th
 **  roots of unity, $0=1+e_n^{n/p}+e_n^{2n/p}+..+e_n^{(p-1)n/p}$ and multiply
 **  it  by $e_n^i$,   $0=e_n^i+e_n^{n/p+i}+e_n^{2n/p+i}+..+e_n^{(p-1)n/p+i}$.
-**  Now we subtract $c$ times the left hand side from 'ResultCyc'.
+**  Now we subtract $c$ times the left hand side from 'TLS(ResultCyc)'.
 **
 **  If $p^2$  does not divide  $n$ then the roots  that are  not in the  base
 **  because of $p$ are those  whose exponent is divisable  by $p$.  But $n/p$
@@ -409,10 +421,10 @@ libGAP_Int             libGAP_LtCycNot (
 **  which remove the roots that are not in the base because of larger primes,
 **  will not add new roots that do not lie in the base because of $p$ again.
 **
-**  For an example, suppose 'ResultCyc' is $e_{45}+e_{45}^5 =: e+e^5$.  $e^5$
+**  For an example, suppose 'TLS(ResultCyc)' is $e_{45}+e_{45}^5 =: e+e^5$.  $e^5$
 **  does  not lie in the  base  because $5  \in 5*[-1,0,1]$  mod $9$ and also
 **  because it is  divisable  by 5.  After  subtracting  $e^5*(1+e_3+e_3^2) =
-**  e^5+e^{20}+e^{35}$ from  'ResultCyc' we get $e-e^{20}-e^{35}$.  Those two
+**  e^5+e^{20}+e^{35}$ from  'TLS(ResultCyc)' we get $e-e^{20}-e^{35}$.  Those two
 **  roots are  still not  in the  base because of  5.  But  after subtracting
 **  $-e^{20}*(1+e_5+e_5^2+e_5^3+e_5^4)=-e^{20}-e^{29}-e^{38}-e^2-e^{11}$  and
 **  $-e^{35}*(1+e_5+e_5^2+e_5^3+e_5^4)=-e^{35}-e^{44}-e^8-e^{17}-e^{26}$   we
@@ -442,7 +454,7 @@ void            libGAP_ConvertToBase (
     libGAP_Obj                 sum;            /* sum of two coefficients         */
 
     /* get a pointer to the cyclotomic and a copy of n to factor           */
-    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
     nn  = n;
 
     /* first handle 2                                                      */
@@ -459,9 +471,9 @@ void            libGAP_ConvertToBase (
                     l = (k + n/2) % n;
                     if ( ! libGAP_ARE_INTOBJS( res[l], res[k] )
                       || ! libGAP_DIFF_INTOBJS( sum, res[l], res[k] ) ) {
-                        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                         sum = libGAP_DIFF( res[l], res[k] );
-                        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                     }
                     res[l] = sum;
                     res[k] = libGAP_INTOBJ_INT(0);
@@ -474,9 +486,9 @@ void            libGAP_ConvertToBase (
                     l = (k + n/2) % n;
                     if ( ! libGAP_ARE_INTOBJS( res[l], res[k] )
                       || ! libGAP_DIFF_INTOBJS( sum, res[l], res[k] ) ) {
-                        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                         sum = libGAP_DIFF( res[l], res[k] );
-                        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                     }
                     res[l] = sum;
                     res[k] = libGAP_INTOBJ_INT(0);
@@ -506,9 +518,9 @@ void            libGAP_ConvertToBase (
                     for ( l = k+n/p; l < k+n; l += n/p ) {
                         if ( ! libGAP_ARE_INTOBJS( res[l%n], res[k] )
                           || ! libGAP_DIFF_INTOBJS( sum, res[l%n], res[k] ) ) {
-                            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                             sum = libGAP_DIFF( res[l%n], res[k] );
-                            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                            res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                         }
                         res[l%n] = sum;
                     }
@@ -522,9 +534,9 @@ void            libGAP_ConvertToBase (
                     for ( l = k+n/p; l < k+n; l += n/p ) {
                         if ( ! libGAP_ARE_INTOBJS( res[l%n], res[k] )
                           || ! libGAP_DIFF_INTOBJS( sum, res[l%n], res[k] ) ) {
-                            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                             sum = libGAP_DIFF( res[l%n], res[k] );
-                            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                            res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                         }
                         res[l%n] = sum;
                     }
@@ -535,7 +547,7 @@ void            libGAP_ConvertToBase (
     }
 
     /* notify Gasman                                                       */
-    libGAP_CHANGED_BAG( libGAP_ResultCyc );
+    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
 }
 
 
@@ -543,10 +555,10 @@ void            libGAP_ConvertToBase (
 **
 *F  Cyclotomic(<n>,<m>) . . . . . . . . . . create a packed cyclotomic, local
 **
-**  'Cyclotomic'    reduces  the cyclotomic   'ResultCyc'   into the smallest
+**  'Cyclotomic'    reduces  the cyclotomic   'TLS(ResultCyc)'   into the smallest
 **  possible cyclotomic subfield and returns it in packed form.
 **
-**  'ResultCyc'  must   also    be already converted      into  the base   by
+**  'TLS(ResultCyc)'  must   also    be already converted      into  the base   by
 **  'ConvertToBase'.   <n> must be  the order of the  primitive root in which
 **  written.
 **
@@ -560,7 +572,7 @@ void            libGAP_ConvertToBase (
 **  rational.  If this is the case 'Cyclotomic' reduces it into the rationals
 **  and returns it as a rational.
 **
-**  After 'Cyclotomic' has  done its work it clears  the 'ResultCyc'  bag, so
+**  After 'Cyclotomic' has  done its work it clears  the 'TLS(ResultCyc)'  bag, so
 **  that it only contains 'INTOBJ_INT(0)'.  Thus the arithmetic functions can
 **  use this buffer without clearing it first.
 **
@@ -592,7 +604,7 @@ libGAP_Obj             libGAP_Cyclotomic (
     static libGAP_UInt         nrp;            /* number of its prime factors     */
 
     /* get a pointer to the cyclotomic and a copy of n to factor           */
-    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
 
     /* count the terms and compute the gcd of the exponents with n         */
     len = 0;
@@ -646,12 +658,12 @@ libGAP_Obj             libGAP_Cyclotomic (
         if ( nrp % 2 == 0 )
             res[0] = cof;
         else {
-            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
             res[0] = libGAP_DIFF( libGAP_INTOBJ_INT(0), cof );
         }
         n = 1;
     }
-    libGAP_CHANGED_BAG( libGAP_ResultCyc );
+    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
 
     /* for all primes $p$ try to reduce from $Q(e_n)$ into $Q(e_{n/p})$    */
     gcd = phi; s = len; while ( s != 0 ) { t = s; s = gcd % s; gcd = t; }
@@ -684,16 +696,16 @@ libGAP_Obj             libGAP_Cyclotomic (
                     res[i] = libGAP_INTOBJ_INT( - libGAP_INT_INTOBJ(cof) );
                     if ( ! libGAP_IS_INTOBJ(cof)
                       || (cof == libGAP_INTOBJ_INT(-(1L<<libGAP_NR_SMALL_INT_BITS))) ) {
-                        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                         cof = libGAP_DIFF( libGAP_INTOBJ_INT(0), cof );
-                        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                         res[i] = cof;
                     }
                     for ( k = i+n/p; k < i+n && eql; k += n/p )
                         res[k%n] = libGAP_INTOBJ_INT(0);
                 }
                 len = len / (p-1);
-                libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
 
                 /* now replace $e_n^{i*p}$ by $e_{n/p}^{i}$                */
                 for ( i = 1; i < n/p; i++ ) {
@@ -714,7 +726,7 @@ libGAP_Obj             libGAP_Cyclotomic (
         res[0] = libGAP_INTOBJ_INT(0);
     }
 
-    /* otherwise copy terms into a new 'T_CYC' bag and clear 'ResultCyc'   */
+    /* otherwise copy terms into a new 'T_CYC' bag and clear 'TLS(ResultCyc)'   */
     else {
         cyc = libGAP_NewBag( libGAP_T_CYC, (len+1)*(sizeof(libGAP_Obj)+sizeof(libGAP_UInt4)) );
         cfs = libGAP_COEFS_CYC(cyc);
@@ -722,7 +734,7 @@ libGAP_Obj             libGAP_Cyclotomic (
         cfs[0] = libGAP_INTOBJ_INT(n);
         exs[0] = 0;
         k = 1;
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         for ( i = 0; i < n; i++ ) {
             if ( res[i] != libGAP_INTOBJ_INT(0) ) {
                 cfs[k] = res[i];
@@ -750,9 +762,8 @@ libGAP_UInt4 libGAP_CyclotomicsLimit = 1000000;
 
 static libGAP_UInt libGAP_FindCommonField(libGAP_UInt nl, libGAP_UInt nr, libGAP_UInt *ml, libGAP_UInt *mr)
 {
-  libGAP_UInt n,i,a,b,c;
+  libGAP_UInt n,a,b,c;
   libGAP_UInt8 n8;
-  libGAP_Obj *res;
   
   /* get the smallest field that contains both cyclotomics               */
   /* First Euclid's Algorithm for gcd */
@@ -787,12 +798,7 @@ static libGAP_UInt libGAP_FindCommonField(libGAP_UInt nl, libGAP_UInt nr, libGAP
   *mr = n/nr;
 
   /* make sure that the result bag is large enough                      */
-  if ( libGAP_LEN_PLIST(libGAP_ResultCyc) < n ) {
-    libGAP_GROW_PLIST( libGAP_ResultCyc, n );
-    libGAP_SET_LEN_PLIST( libGAP_ResultCyc, n );
-    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
-    for ( i = 0; i < n; i++ ) { res[i] = libGAP_INTOBJ_INT(0); }
-  }
+  libGAP_GrowResultCyc(n);
   return n;
 }
 
@@ -871,15 +877,15 @@ libGAP_Obj             libGAP_SumCyc (
  
     /* Copy the left operand into the result                               */
     if ( libGAP_TNUM_OBJ(opL) != libGAP_T_CYC ) {
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         res[0] = opL;
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
     else {
         len = libGAP_SIZE_CYC(opL);
         cfs = libGAP_COEFS_CYC(opL);
         exs = libGAP_EXPOS_CYC(opL,len);
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         if ( ml == 1 ) {
             for ( i = 1; i < len; i++ )
                 res[exs[i]] = cfs[i];
@@ -888,34 +894,34 @@ libGAP_Obj             libGAP_SumCyc (
             for ( i = 1; i < len; i++ )
                 res[exs[i]*ml] = cfs[i];
         }
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
 
     /* add the right operand to the result                                 */
     if ( libGAP_TNUM_OBJ(opR) != libGAP_T_CYC ) {
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         sum = libGAP_SUM( res[0], opR );
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         res[0] = sum;
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
     else {
         len = libGAP_SIZE_CYC(opR);
         cfs = libGAP_COEFS_CYC(opR);
         exs = libGAP_EXPOS_CYC(opR,len);
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         for ( i = 1; i < len; i++ ) {
             if ( ! libGAP_ARE_INTOBJS( res[exs[i]*mr], cfs[i] )
               || ! libGAP_SUM_INTOBJS( sum, res[exs[i]*mr], cfs[i] ) ) {
-                libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                 sum = libGAP_SUM( res[exs[i]*mr], cfs[i] );
                 cfs = libGAP_COEFS_CYC(opR);
                 exs = libGAP_EXPOS_CYC(opR,len);
-                res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             }
             res[exs[i]*mr] = sum;
         }
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
 
     /* return the base reduced packed cyclotomic                           */
@@ -1015,15 +1021,15 @@ libGAP_Obj             libGAP_DiffCyc (
 
     /* copy the left operand into the result                               */
     if ( libGAP_TNUM_OBJ(opL) != libGAP_T_CYC ) {
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         res[0] = opL;
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
     else {
         len = libGAP_SIZE_CYC(opL);
         cfs = libGAP_COEFS_CYC(opL);
         exs = libGAP_EXPOS_CYC(opL,len);
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         if ( ml == 1 ) {
             for ( i = 1; i < len; i++ )
                 res[exs[i]] = cfs[i];
@@ -1032,34 +1038,34 @@ libGAP_Obj             libGAP_DiffCyc (
             for ( i = 1; i < len; i++ )
                 res[exs[i]*ml] = cfs[i];
         }
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
 
     /* subtract the right operand from the result                          */
     if ( libGAP_TNUM_OBJ(opR) != libGAP_T_CYC ) {
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         sum = libGAP_DIFF( res[0], opR );
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         res[0] = sum;
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
     else {
         len = libGAP_SIZE_CYC(opR);
         cfs = libGAP_COEFS_CYC(opR);
         exs = libGAP_EXPOS_CYC(opR,len);
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         for ( i = 1; i < len; i++ ) {
             if ( ! libGAP_ARE_INTOBJS( res[exs[i]*mr], cfs[i] )
               || ! libGAP_DIFF_INTOBJS( sum, res[exs[i]*mr], cfs[i] ) ) {
-                libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                 sum = libGAP_DIFF( res[exs[i]*mr], cfs[i] );
                 cfs = libGAP_COEFS_CYC(opR);
                 exs = libGAP_EXPOS_CYC(opR,len);
-                res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             }
             res[exs[i]*mr] = sum;
         }
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     }
 
     /* return the base reduced packed cyclotomic                           */
@@ -1221,19 +1227,19 @@ libGAP_Obj             libGAP_ProdCyc (
             len = libGAP_SIZE_CYC(opL);
             cfs = libGAP_COEFS_CYC(opL);
             exs = libGAP_EXPOS_CYC(opL,len);
-            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+            res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             for ( i = 1; i < len; i++ ) {
                 if ( ! libGAP_ARE_INTOBJS( res[(e+exs[i]*ml)%n], cfs[i] )
                   || ! libGAP_SUM_INTOBJS( sum, res[(e+exs[i]*ml)%n], cfs[i] ) ) {
-                    libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                     sum = libGAP_SUM( res[(e+exs[i]*ml)%n], cfs[i] );
                     cfs = libGAP_COEFS_CYC(opL);
                     exs = libGAP_EXPOS_CYC(opL,len);
-                    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                 }
                 res[(e+exs[i]*ml)%n] = sum;
             }
-            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
         }
 
         /* if the coefficient is -1 just subtract                          */
@@ -1241,19 +1247,19 @@ libGAP_Obj             libGAP_ProdCyc (
             len = libGAP_SIZE_CYC(opL);
             cfs = libGAP_COEFS_CYC(opL);
             exs = libGAP_EXPOS_CYC(opL,len);
-            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+            res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             for ( i = 1; i < len; i++ ) {
                 if ( ! libGAP_ARE_INTOBJS( res[(e+exs[i]*ml)%n], cfs[i] )
                   || ! libGAP_DIFF_INTOBJS( sum, res[(e+exs[i]*ml)%n], cfs[i] ) ) {
-                    libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                     sum = libGAP_DIFF( res[(e+exs[i]*ml)%n], cfs[i] );
                     cfs = libGAP_COEFS_CYC(opL);
                     exs = libGAP_EXPOS_CYC(opL,len);
-                    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                 }
                 res[(e+exs[i]*ml)%n] = sum;
             }
-            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
         }
 
         /* if the coefficient is a small integer use immediate operations  */
@@ -1261,40 +1267,40 @@ libGAP_Obj             libGAP_ProdCyc (
             len = libGAP_SIZE_CYC(opL);
             cfs = libGAP_COEFS_CYC(opL);
             exs = libGAP_EXPOS_CYC(opL,len);
-            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+            res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             for ( i = 1; i < len; i++ ) {
                 if ( ! libGAP_ARE_INTOBJS( cfs[i], res[(e+exs[i]*ml)%n] )
                   || ! libGAP_PROD_INTOBJS( prd, cfs[i], c )
                   || ! libGAP_SUM_INTOBJS( sum, res[(e+exs[i]*ml)%n], prd ) ) {
-                    libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                     prd = libGAP_PROD( cfs[i], c );
                     exs = libGAP_EXPOS_CYC(opL,len);
-                    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                     sum = libGAP_SUM( res[(e+exs[i]*ml)%n], prd );
                     cfs = libGAP_COEFS_CYC(opL);
                     exs = libGAP_EXPOS_CYC(opL,len);
-                    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                 }
                 res[(e+exs[i]*ml)%n] = sum;
             }
-            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
         }
 
         /* otherwise do it the normal way                                  */
         else {
             len = libGAP_SIZE_CYC(opL);
             for ( i = 1; i < len; i++ ) {
-                libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                 cfs = libGAP_COEFS_CYC(opL);
                 prd = libGAP_PROD( cfs[i], c );
                 exs = libGAP_EXPOS_CYC(opL,len);
-                res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                 sum = libGAP_SUM( res[(e+exs[i]*ml)%n], prd );
                 exs = libGAP_EXPOS_CYC(opL,len);
-                res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
                 res[(e+exs[i]*ml)%n] = sum;
             }
-            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
         }
 
     }
@@ -1363,10 +1369,10 @@ libGAP_Obj             libGAP_InvCyc (
             /* permute the terms                                           */
             cfs = libGAP_COEFS_CYC(op);
             exs = libGAP_EXPOS_CYC(op,len);
-            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+            res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             for ( k = 1; k < len; k++ )
                 res[(i*exs[k])%n] = cfs[k];
-            libGAP_CHANGED_BAG( libGAP_ResultCyc );
+            libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
 
             /* if n is squarefree conversion and reduction are unnecessary */
             if ( n < sqr*sqr ) {
@@ -1418,12 +1424,12 @@ libGAP_Obj             libGAP_PowCyc (
     }
 
     /* for $e_n^exp$ just put a 1 at the <exp>th position and convert      */
-    else if ( opL == libGAP_LastECyc ) {
-        exp = (exp % libGAP_LastNCyc + libGAP_LastNCyc) % libGAP_LastNCyc;
-        libGAP_SET_ELM_PLIST( libGAP_ResultCyc, exp, libGAP_INTOBJ_INT(1) );
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
-        libGAP_ConvertToBase( libGAP_LastNCyc );
-        pow = libGAP_Cyclotomic( libGAP_LastNCyc, 1 );
+    else if ( opL == libGAP_TLS(libGAP_LastECyc) ) {
+        exp = (exp % libGAP_TLS(libGAP_LastNCyc) + libGAP_TLS(libGAP_LastNCyc)) % libGAP_TLS(libGAP_LastNCyc);
+        libGAP_SET_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), exp, libGAP_INTOBJ_INT(1) );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
+        libGAP_ConvertToBase( libGAP_TLS(libGAP_LastNCyc) );
+        pow = libGAP_Cyclotomic( libGAP_TLS(libGAP_LastNCyc), 1 );
     }
 
     /* for $(c*e_n^i)^exp$ if $e_n^i$ belongs to the base put 1 at $i*exp$ */
@@ -1431,9 +1437,9 @@ libGAP_Obj             libGAP_PowCyc (
         n = libGAP_INT_INTOBJ( libGAP_NOF_CYC(opL) );
         pow = libGAP_POW( libGAP_COEFS_CYC(opL)[1], opR );
         i = libGAP_EXPOS_CYC(opL,2)[1];
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         res[((exp*i)%n+n)%n] = pow;
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
         libGAP_ConvertToBase( n );
         pow = libGAP_Cyclotomic( n, 1 );
     }
@@ -1479,7 +1485,6 @@ libGAP_Obj libGAP_FuncE (
     libGAP_Obj                 self,
     libGAP_Obj                 n )
 {
-    libGAP_UInt                i;              /* loop variable                   */
     libGAP_Obj *               res;            /* pointer into result bag         */
 
     /* do full operation                                                   */
@@ -1502,23 +1507,18 @@ libGAP_Obj libGAP_FuncE (
         return libGAP_INTOBJ_INT(-1);
 
     /* if the root is not known already construct it                       */
-    if ( libGAP_LastNCyc != libGAP_INT_INTOBJ(n) ) {
-        libGAP_LastNCyc = libGAP_INT_INTOBJ(n);
-        if ( libGAP_LEN_PLIST(libGAP_ResultCyc) < libGAP_LastNCyc ) {
-            libGAP_GROW_PLIST( libGAP_ResultCyc, libGAP_LastNCyc );
-            libGAP_SET_LEN_PLIST( libGAP_ResultCyc, libGAP_LastNCyc );
-            res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
-            for ( i = 0; i < libGAP_LastNCyc; i++ ) { res[i] = libGAP_INTOBJ_INT(0); }
-        }
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+    if ( libGAP_TLS(libGAP_LastNCyc) != libGAP_INT_INTOBJ(n) ) {
+        libGAP_TLS(libGAP_LastNCyc) = libGAP_INT_INTOBJ(n);
+        libGAP_GrowResultCyc(libGAP_TLS(libGAP_LastNCyc));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         res[1] = libGAP_INTOBJ_INT(1);
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
-        libGAP_ConvertToBase( libGAP_LastNCyc );
-        libGAP_LastECyc = libGAP_Cyclotomic( libGAP_LastNCyc, 1 );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
+        libGAP_ConvertToBase( libGAP_TLS(libGAP_LastNCyc) );
+        libGAP_TLS(libGAP_LastECyc) = libGAP_Cyclotomic( libGAP_TLS(libGAP_LastNCyc), 1 );
     }
 
     /* return the root                                                     */
-    return libGAP_LastECyc;
+    return libGAP_TLS(libGAP_LastECyc);
 }
 
 
@@ -1880,11 +1880,11 @@ libGAP_Obj libGAP_FuncGALOIS_CYC (
         len = libGAP_SIZE_CYC(cyc);
         cfs = libGAP_COEFS_CYC(cyc);
         exs = libGAP_EXPOS_CYC(cyc,len);
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         for ( i = 1; i < len; i++ ) {
             res[(libGAP_UInt8)exs[i]*(libGAP_UInt8)o%(libGAP_UInt8)n] = cfs[i];
         }
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
 
         /* if n is squarefree conversion and reduction are unnecessary     */
         if ( n < sqr*sqr || (o == n-1 && n % 2 != 0) ) {
@@ -1904,19 +1904,19 @@ libGAP_Obj libGAP_FuncGALOIS_CYC (
         len = libGAP_SIZE_CYC(cyc);
         cfs = libGAP_COEFS_CYC(cyc);
         exs = libGAP_EXPOS_CYC(cyc,len);
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+        res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
         for ( i = 1; i < len; i++ ) {
             if ( ! libGAP_ARE_INTOBJS( res[(libGAP_UInt8)exs[i]*(libGAP_UInt8)o%(libGAP_UInt8)n], cfs[i] )
               || ! libGAP_SUM_INTOBJS( sum, res[(libGAP_UInt8)exs[i]*(libGAP_UInt8)o%(libGAP_UInt8)n], cfs[i] ) ) {
-                libGAP_CHANGED_BAG( libGAP_ResultCyc );
+                libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
                 sum = libGAP_SUM( res[(libGAP_UInt8)exs[i]*(libGAP_UInt8)o%(libGAP_UInt8)n], cfs[i] );
                 cfs = libGAP_COEFS_CYC(cyc);
                 exs = libGAP_EXPOS_CYC(cyc,len);
-                res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+                res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
             }
             res[exs[i]*o%n] = sum;
         }
-        libGAP_CHANGED_BAG( libGAP_ResultCyc );
+        libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
 
         /* if n is squarefree conversion and reduction are unnecessary     */
         if ( n < sqr*sqr ) {
@@ -1969,15 +1969,10 @@ libGAP_Obj libGAP_FuncCycList (
 
     /* enlarge the buffer if necessary                                     */
     n = libGAP_LEN_PLIST( list );
-    if ( libGAP_LEN_PLIST(libGAP_ResultCyc) < n ) {
-        libGAP_GROW_PLIST( libGAP_ResultCyc, n );
-        libGAP_SET_LEN_PLIST( libGAP_ResultCyc, n );
-        res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
-        for ( i = 0; i < n; i++ ) { res[i] = libGAP_INTOBJ_INT(0); }
-    }
+    libGAP_GrowResultCyc(n);
 
     /* transfer the coefficients into the buffer                           */
-    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
     for ( i = 0; i < n; i++ ) {
         val = libGAP_ELM_PLIST( list, i+1 );
         if ( ! ( libGAP_TNUM_OBJ(val) == libGAP_T_INT ||
@@ -1991,7 +1986,7 @@ libGAP_Obj libGAP_FuncCycList (
     }
 
     /* return the base reduced packed cyclotomic                           */
-    libGAP_CHANGED_BAG( libGAP_ResultCyc );
+    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_ResultCyc) );
     libGAP_ConvertToBase( n );
     return libGAP_Cyclotomic( n, 1 );
 }
@@ -2150,8 +2145,8 @@ static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
 static libGAP_Int libGAP_InitKernel (
     libGAP_StructInitInfo *    libGAP_module )
 {
-  libGAP_LastECyc = (libGAP_Obj)0;
-  libGAP_LastNCyc = 0;
+    libGAP_TLS(libGAP_LastECyc) = (libGAP_Obj)0;
+    libGAP_TLS(libGAP_LastNCyc) = 0;
   
     /* install the marking function                                        */
     libGAP_InfoBags[ libGAP_T_CYC ].name = "cyclotomic";
@@ -2163,7 +2158,7 @@ static libGAP_Int libGAP_InitKernel (
     /* tell Gasman about the place were we remember the primitive root     */
     libGAP_InitGlobalBag( &libGAP_LastECyc, "src/cyclotom.c:LastECyc" );
 
-    /* install the kind function                                           */
+    /* install the type function                                           */
     libGAP_ImportGVarFromLibrary( "TYPE_CYC", &libGAP_TYPE_CYC );
     libGAP_TypeObjFuncs[ libGAP_T_CYC ] = libGAP_TypeCyc;
 
@@ -2247,9 +2242,9 @@ static libGAP_Int libGAP_InitLibrary (
     libGAP_UInt                i;              /* loop variable                   */
 
     /* create the result buffer                                            */
-    libGAP_ResultCyc = libGAP_NEW_PLIST( libGAP_T_PLIST, 1024 );
-    libGAP_SET_LEN_PLIST( libGAP_ResultCyc, 1024 );
-    res = &(libGAP_ELM_PLIST( libGAP_ResultCyc, 1 ));
+    libGAP_TLS(libGAP_ResultCyc) = libGAP_NEW_PLIST( libGAP_T_PLIST, 1024 );
+    libGAP_SET_LEN_PLIST( libGAP_TLS(libGAP_ResultCyc), 1024 );
+    res = &(libGAP_ELM_PLIST( libGAP_TLS(libGAP_ResultCyc), 1 ));
     for ( i = 0; i < 1024; i++ ) { res[i] = libGAP_INTOBJ_INT(0); }
 
     /* init filters and functions                                          */
@@ -2284,7 +2279,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoCyc ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
