@@ -32,7 +32,6 @@
 #include        "calls.h"               /* generic call mechanism          */
 
 #include        "code.h"                /* coder                           */
-#include        "vars.h"                /* variables                       */
 #include        "exprs.h"               /* expressions                     */
 #include        "stats.h"               /* statements                      */
 
@@ -50,7 +49,13 @@
 
 #include        "opers.h"               /* generic operations              */
 #include        "gvars.h"
+#include        "thread.h"              /* threads                         */
+#include        "tls.h"                 /* thread-local storage            */
 
+#include        "vars.h"                /* variables                       */
+
+
+#include        "profile.h"             /* installing methods              */
 /****************************************************************************
 **
 *F ExecProccallOpts( <call> ). . execute a procedure call with options
@@ -140,7 +145,7 @@ libGAP_UInt            libGAP_ExecProccall0args (
     else {
       libGAP_CALL_0ARGS( func );
     }
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -168,7 +173,7 @@ libGAP_UInt            libGAP_ExecProccall1args (
       libGAP_SET_BRK_CALL_TO( call );
       libGAP_CALL_1ARGS( func, arg1 );
     } 
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -198,7 +203,7 @@ libGAP_UInt            libGAP_ExecProccall2args (
       libGAP_SET_BRK_CALL_TO( call );
       libGAP_CALL_2ARGS( func, arg1, arg2 );
     }
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -230,7 +235,7 @@ libGAP_UInt            libGAP_ExecProccall3args (
       libGAP_SET_BRK_CALL_TO( call );
       libGAP_CALL_3ARGS( func, arg1, arg2, arg3 );
     }
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -264,7 +269,7 @@ libGAP_UInt            libGAP_ExecProccall4args (
       libGAP_SET_BRK_CALL_TO( call );
       libGAP_CALL_4ARGS( func, arg1, arg2, arg3, arg4 );
     }
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -306,7 +311,7 @@ libGAP_UInt            libGAP_ExecProccall5args (
       libGAP_SET_BRK_CALL_TO( call );
       libGAP_CALL_5ARGS( func, arg1, arg2, arg3, arg4, arg5 );
     }
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -344,7 +349,7 @@ libGAP_UInt            libGAP_ExecProccall6args (
       libGAP_SET_BRK_CALL_TO( call );
       libGAP_CALL_6ARGS( func, arg1, arg2, arg3, arg4, arg5, arg6 );
     }
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -382,7 +387,7 @@ libGAP_UInt            libGAP_ExecProccallXargs (
       libGAP_CALL_XARGS( func, args );
     }
 
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -448,7 +453,7 @@ libGAP_Obj             libGAP_EvalFunccall0args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_0ARGS( func );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -480,7 +485,7 @@ libGAP_Obj             libGAP_EvalFunccall1args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_1ARGS( func, arg1 );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -515,7 +520,7 @@ libGAP_Obj             libGAP_EvalFunccall2args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_2ARGS( func, arg1, arg2 );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -552,7 +557,7 @@ libGAP_Obj             libGAP_EvalFunccall3args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_3ARGS( func, arg1, arg2, arg3 );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -590,7 +595,7 @@ libGAP_Obj             libGAP_EvalFunccall4args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_4ARGS( func, arg1, arg2, arg3, arg4 );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -631,7 +636,7 @@ libGAP_Obj             libGAP_EvalFunccall5args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_5ARGS( func, arg1, arg2, arg3, arg4, arg5 );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -674,7 +679,7 @@ libGAP_Obj             libGAP_EvalFunccall6args (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_6ARGS( func, arg1, arg2, arg3, arg4, arg5, arg6 );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -715,7 +720,7 @@ libGAP_Obj             libGAP_EvalFunccallXargs (
     /* call the function and return the result                             */
     libGAP_SET_BRK_CALL_TO( call );
     result = libGAP_CALL_XARGS( func, args );
-    if (libGAP_UserHasQuit || libGAP_UserHasQUIT) /* the procedure must have called
+    if (libGAP_TLS(libGAP_UserHasQuit) || libGAP_TLS(libGAP_UserHasQUIT)) /* the procedure must have called
                                        READ() and the user quit from a break
                                        loop inside it */
       libGAP_ReadEvalError();
@@ -751,7 +756,7 @@ libGAP_Obj             libGAP_EvalFunccallXargs (
 **  'DoExecFunc<i>args' first switches  to a new  values bag.  Then it enters
 **  the arguments <arg1>, <arg2>, and so on in this new  values bag.  Then it
 **  executes  the function body.   After  that it  switches back  to  the old
-**  values bag.  Finally it returns the result from 'ReturnObjStat'.
+**  values bag.  Finally it returns the result from 'TLS(ReturnObjStat)'.
 **
 **  Note that these functions are never called directly, they are only called
 **  through the function call mechanism.
@@ -770,41 +775,52 @@ static void libGAP_RecursionDepthTrap( void )
      * when quit-ting a higher level brk-loop to a lower level one.
      * Therefore we don't do anything if  RecursionDepth <= 0
     */
-    if (libGAP_RecursionDepth > 0) {
-        recursionDepth = libGAP_RecursionDepth;
-        libGAP_RecursionDepth = 0;
+    if (libGAP_TLS(libGAP_RecursionDepth) > 0) {
+        recursionDepth = libGAP_TLS(libGAP_RecursionDepth);
+        libGAP_TLS(libGAP_RecursionDepth) = 0;
         libGAP_ErrorReturnVoid( "recursion depth trap (%d)\n",         
                          (libGAP_Int)recursionDepth, 0L,               
                          "you may 'return;'" );
-        libGAP_RecursionDepth = recursionDepth;
+        libGAP_TLS(libGAP_RecursionDepth) = recursionDepth;
     }
 }
      
 static inline void libGAP_CheckRecursionBefore( void )
 {
-    libGAP_RecursionDepth++;                                           
+    libGAP_TLS(libGAP_RecursionDepth)++;                                           
     if ( libGAP_RecursionTrapInterval &&                                
-         0 == (libGAP_RecursionDepth % libGAP_RecursionTrapInterval) )
+         0 == (libGAP_TLS(libGAP_RecursionDepth) % libGAP_RecursionTrapInterval) )
       libGAP_RecursionDepthTrap();
 }
 
 
- libGAP_Obj libGAP_STEVES_TRACING;
+libGAP_Obj libGAP_STEVES_TRACING;
 
-#define libGAP_CHECK_RECURSION_BEFORE libGAP_CheckRecursionBefore();
+#define libGAP_CHECK_RECURSION_BEFORE \
+            libGAP_CheckRecursionBefore(); \
+            libGAP_ProfileLineByLineIntoFunction(func);
 
+#define libGAP_CHECK_RECURSION_AFTER \
+            libGAP_TLS(libGAP_RecursionDepth)--; \
+            libGAP_ProfileLineByLineOutFunction(func);
 
-#define libGAP_CHECK_RECURSION_AFTER     libGAP_RecursionDepth--;       
+#define libGAP_REMEMBER_LOCKSTACK() \
+    do { } while (0)
+
+#define libGAP_CLEAR_LOCK_STACK() \
+    do { } while (0)
 
 
 libGAP_Obj libGAP_DoExecFunc0args (
     libGAP_Obj                 func )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
 
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
+    
 
     /* switch to a new values bag                                          */
     libGAP_SWITCH_TO_NEW_LVARS( func, 0, libGAP_NLOC_FUNC(func), oldLvars );
@@ -813,21 +829,22 @@ libGAP_Obj libGAP_DoExecFunc0args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* Switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
-
+    
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -837,6 +854,7 @@ libGAP_Obj             libGAP_DoExecFunc1args (
     libGAP_Obj                 arg1 )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
@@ -851,21 +869,22 @@ libGAP_Obj             libGAP_DoExecFunc1args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
 
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -876,6 +895,7 @@ libGAP_Obj             libGAP_DoExecFunc2args (
     libGAP_Obj                 arg2 )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
@@ -891,21 +911,22 @@ libGAP_Obj             libGAP_DoExecFunc2args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
 
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -917,6 +938,7 @@ libGAP_Obj             libGAP_DoExecFunc3args (
     libGAP_Obj                 arg3 )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
@@ -933,21 +955,22 @@ libGAP_Obj             libGAP_DoExecFunc3args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
 
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -960,6 +983,7 @@ libGAP_Obj             libGAP_DoExecFunc4args (
     libGAP_Obj                 arg4 )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
@@ -977,21 +1001,22 @@ libGAP_Obj             libGAP_DoExecFunc4args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
 
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -1005,6 +1030,7 @@ libGAP_Obj             libGAP_DoExecFunc5args (
     libGAP_Obj                 arg5 )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
@@ -1023,21 +1049,22 @@ libGAP_Obj             libGAP_DoExecFunc5args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
 
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -1052,6 +1079,7 @@ libGAP_Obj             libGAP_DoExecFunc6args (
     libGAP_Obj                 arg6 )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
 
     libGAP_CHECK_RECURSION_BEFORE
@@ -1071,21 +1099,22 @@ libGAP_Obj             libGAP_DoExecFunc6args (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
     libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
 
     /* switch back to the old values bag                                   */
-    libGAP_SWITCH_TO_OLD_LVARS( oldLvars );
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
 
     libGAP_CHECK_RECURSION_AFTER
 
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
       }
 }
@@ -1095,6 +1124,7 @@ libGAP_Obj             libGAP_DoExecFuncXargs (
     libGAP_Obj                 args )
 {
     libGAP_Bag                 oldLvars;       /* old values bag                  */
+    libGAP_REMEMBER_LOCKSTACK();
     libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
     libGAP_UInt                len;            /* number of arguments             */
     libGAP_UInt                i;              /* loop variable                   */
@@ -1123,6 +1153,61 @@ libGAP_Obj             libGAP_DoExecFuncXargs (
     libGAP_REM_BRK_CURR_STAT();
     libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
     libGAP_RES_BRK_CURR_STAT();
+    libGAP_CLEAR_LOCK_STACK();
+
+   /* remove the link to the calling function, in case this values bag
+       stays alive due to higher variable reference */
+    libGAP_SET_BRK_CALL_FROM( ((libGAP_Obj) 0));
+
+    /* switch back to the old values bag                                   */
+    libGAP_SWITCH_TO_OLD_LVARS_AND_FREE( oldLvars );
+
+    libGAP_CHECK_RECURSION_AFTER
+
+    /* return the result                                                   */
+      {
+        libGAP_Obj                 returnObjStat;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
+        return returnObjStat;
+      }
+}
+
+
+
+libGAP_Obj libGAP_DoPartialUnWrapFunc(libGAP_Obj func, libGAP_Obj args) {
+  
+  libGAP_Bag                 oldLvars;       /* old values bag                  */
+  libGAP_OLD_BRK_CURR_STAT                   /* old executing statement         */
+    libGAP_UInt                named;            /* number of arguments             */
+  libGAP_UInt                i;              /* loop variable                   */
+  libGAP_UInt len;
+  libGAP_Obj argx;
+
+
+      named = ((libGAP_UInt)-libGAP_NARG_FUNC(func))-1;
+    len = libGAP_LEN_PLIST(args);
+
+    if (named > len) { /* Can happen for > 6 arguments */
+      argx = libGAP_NargError(func, len);
+      return libGAP_DoOperation2Args(libGAP_CallFuncListOper, func, argx);
+    }
+
+    libGAP_CHECK_RECURSION_BEFORE    
+    libGAP_SWITCH_TO_NEW_LVARS( func, named+1, libGAP_NLOC_FUNC(func), oldLvars );
+
+    for (i = 1; i <= named; i++) {
+      libGAP_ASS_LVAR(i, libGAP_ELM_PLIST(args,i));
+    }
+    for (i = named+1; i <= len; i++) {
+    libGAP_SET_ELM_PLIST(args, i-named, libGAP_ELM_PLIST(args,i));
+  }
+  libGAP_SET_LEN_PLIST(args, len-named);
+  libGAP_ASS_LVAR(named+1, args);
+    /* execute the statement sequence                                      */
+    libGAP_REM_BRK_CURR_STAT();
+    libGAP_EXEC_STAT( libGAP_FIRST_STAT_CURR_FUNC );
+    libGAP_RES_BRK_CURR_STAT();
 
    /* remove the link to the calling function, in case this values bag
        stays alive due to higher variable reference */
@@ -1136,13 +1221,12 @@ libGAP_Obj             libGAP_DoExecFuncXargs (
     /* return the result                                                   */
       {
         libGAP_Obj                 returnObjStat;
-        returnObjStat = libGAP_ReturnObjStat;
-        libGAP_ReturnObjStat = (libGAP_Obj)0;
+        returnObjStat = libGAP_TLS(libGAP_ReturnObjStat);
+        libGAP_TLS(libGAP_ReturnObjStat) = (libGAP_Obj)0;
         return returnObjStat;
-      }
+      }  
 }
 
-
 /****************************************************************************
 **
 *F  MakeFunction(<fexp>)  . . . . . . . . . . . . . . . . . . make a function
@@ -1164,7 +1248,8 @@ libGAP_Obj             libGAP_MakeFunction (
     else if ( libGAP_NARG_FUNC(fexp) ==  5 )  hdlr = libGAP_DoExecFunc5args;
     else if ( libGAP_NARG_FUNC(fexp) ==  6 )  hdlr = libGAP_DoExecFunc6args;
     else if ( libGAP_NARG_FUNC(fexp) >=  7 )  hdlr = libGAP_DoExecFuncXargs;
-    else   /* NARG_FUNC(fexp) == -1 */ hdlr = libGAP_DoExecFunc1args;
+    else if ( libGAP_NARG_FUNC(fexp) == -1 )  hdlr = libGAP_DoExecFunc1args;
+    else /* NARG_FUNC(fexp) < -1 */    hdlr = libGAP_DoPartialUnWrapFunc;
 
     /* make the function                                                   */
     func = libGAP_NewFunctionT( libGAP_T_FUNCTION, libGAP_SIZE_FUNC,
@@ -1175,9 +1260,9 @@ libGAP_Obj             libGAP_MakeFunction (
     /* install the things an interpreted function needs                    */
     libGAP_NLOC_FUNC( func ) = libGAP_NLOC_FUNC( fexp );
     libGAP_BODY_FUNC( func ) = libGAP_BODY_FUNC( fexp );
-    libGAP_ENVI_FUNC( func ) = libGAP_CurrLVars;
-    /* the 'CHANGED_BAG(CurrLVars)' is needed because it is delayed        */
-    libGAP_CHANGED_BAG( libGAP_CurrLVars );
+    libGAP_ENVI_FUNC( func ) = libGAP_TLS(libGAP_CurrLVars);
+    /* the 'CHANGED_BAG(TLS(CurrLVars))' is needed because it is delayed        */
+    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_CurrLVars) );
     libGAP_FEXS_FUNC( func ) = libGAP_FEXS_FUNC( fexp );
 
     /* return the function                                                 */
@@ -1318,12 +1403,12 @@ void            libGAP_ExecBegin ( libGAP_Obj frame )
     /* remember the old execution state                                    */
     execState = libGAP_NewBag( libGAP_T_PLIST, 4*sizeof(libGAP_Obj) );
     libGAP_ADDR_OBJ(execState)[0] = (libGAP_Obj)3;
-    libGAP_ADDR_OBJ(execState)[1] = libGAP_ExecState;
-    libGAP_ADDR_OBJ(execState)[2] = libGAP_CurrLVars;
-    /* the 'CHANGED_BAG(CurrLVars)' is needed because it is delayed        */
-    libGAP_CHANGED_BAG( libGAP_CurrLVars );
-    libGAP_ADDR_OBJ(execState)[3] = libGAP_INTOBJ_INT((libGAP_Int)libGAP_CurrStat);
-    libGAP_ExecState = execState;
+    libGAP_ADDR_OBJ(execState)[1] = libGAP_TLS(libGAP_ExecState);
+    libGAP_ADDR_OBJ(execState)[2] = libGAP_TLS(libGAP_CurrLVars);
+    /* the 'CHANGED_BAG(TLS(CurrLVars))' is needed because it is delayed        */
+    libGAP_CHANGED_BAG( libGAP_TLS(libGAP_CurrLVars) );
+    libGAP_ADDR_OBJ(execState)[3] = libGAP_INTOBJ_INT((libGAP_Int)libGAP_TLS(libGAP_CurrStat));
+    libGAP_TLS(libGAP_ExecState) = execState;
 
     /* set up new state                                                    */
     libGAP_SWITCH_TO_OLD_LVARS( frame );
@@ -1337,12 +1422,12 @@ void            libGAP_ExecEnd (
     if ( ! error ) {
 
         /* the state must be primal again                                  */
-        assert( libGAP_CurrStat  == 0 );
+        assert( libGAP_TLS(libGAP_CurrStat)  == 0 );
 
         /* switch back to the old state                                    */
-        libGAP_SET_BRK_CURR_STAT( (libGAP_Stat)libGAP_INT_INTOBJ((libGAP_ADDR_OBJ(libGAP_ExecState)[3]) ));
-        libGAP_SWITCH_TO_OLD_LVARS( libGAP_ADDR_OBJ(libGAP_ExecState)[2] );
-        libGAP_ExecState = libGAP_ADDR_OBJ(libGAP_ExecState)[1];
+        libGAP_SET_BRK_CURR_STAT( (libGAP_Stat)libGAP_INT_INTOBJ((libGAP_ADDR_OBJ(libGAP_TLS(libGAP_ExecState))[3]) ));
+        libGAP_SWITCH_TO_OLD_LVARS( libGAP_ADDR_OBJ(libGAP_TLS(libGAP_ExecState))[2] );
+        libGAP_TLS(libGAP_ExecState) = libGAP_ADDR_OBJ(libGAP_TLS(libGAP_ExecState))[1];
 
     }
 
@@ -1350,9 +1435,9 @@ void            libGAP_ExecEnd (
     else {
 
         /* switch back to the old state                                    */
-        libGAP_SET_BRK_CURR_STAT( (libGAP_Stat)libGAP_INT_INTOBJ((libGAP_ADDR_OBJ(libGAP_ExecState)[3]) ));
-        libGAP_SWITCH_TO_OLD_LVARS( libGAP_ADDR_OBJ(libGAP_ExecState)[2] );
-        libGAP_ExecState = libGAP_ADDR_OBJ(libGAP_ExecState)[1];
+        libGAP_SET_BRK_CURR_STAT( (libGAP_Stat)libGAP_INT_INTOBJ((libGAP_ADDR_OBJ(libGAP_TLS(libGAP_ExecState))[3]) ));
+        libGAP_SWITCH_TO_OLD_LVARS( libGAP_ADDR_OBJ(libGAP_TLS(libGAP_ExecState))[2] );
+        libGAP_TLS(libGAP_ExecState) = libGAP_ADDR_OBJ(libGAP_TLS(libGAP_ExecState))[1];
 
     }
 }
@@ -1417,8 +1502,8 @@ static libGAP_Int libGAP_InitLibrary (
 static libGAP_Int libGAP_InitKernel (
     libGAP_StructInitInfo *    libGAP_module )
 {
-  libGAP_RecursionTrapInterval = 5000;
-  libGAP_InitCopyGVar("STEVES_TRACING", &libGAP_STEVES_TRACING);
+    libGAP_RecursionTrapInterval = 5000;
+    libGAP_InitCopyGVar("STEVES_TRACING", &libGAP_STEVES_TRACING);
   
     /* make the global variable known to Gasman                            */
     libGAP_InitGlobalBag( &libGAP_ExecState, "src/funcs.c:ExecState" );
@@ -1439,49 +1524,50 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_InitHandlerFunc( libGAP_DoExecFunc5args, "i5");
     libGAP_InitHandlerFunc( libGAP_DoExecFunc6args, "i6");
     libGAP_InitHandlerFunc( libGAP_DoExecFuncXargs, "iX");
+    libGAP_InitHandlerFunc( libGAP_DoPartialUnWrapFunc, "pUW");
 
     /* install the evaluators and executors                                */
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_0ARGS ] = libGAP_ExecProccall0args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_1ARGS ] = libGAP_ExecProccall1args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_2ARGS ] = libGAP_ExecProccall2args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_3ARGS ] = libGAP_ExecProccall3args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_4ARGS ] = libGAP_ExecProccall4args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_5ARGS ] = libGAP_ExecProccall5args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_6ARGS ] = libGAP_ExecProccall6args;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_XARGS ] = libGAP_ExecProccallXargs;
-    libGAP_ExecStatFuncs [ libGAP_T_PROCCALL_OPTS  ] = libGAP_ExecProccallOpts;
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_0ARGS , libGAP_ExecProccall0args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_1ARGS , libGAP_ExecProccall1args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_2ARGS , libGAP_ExecProccall2args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_3ARGS , libGAP_ExecProccall3args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_4ARGS , libGAP_ExecProccall4args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_5ARGS , libGAP_ExecProccall5args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_6ARGS , libGAP_ExecProccall6args);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_XARGS , libGAP_ExecProccallXargs);
+    libGAP_InstallExecStatFunc( libGAP_T_PROCCALL_OPTS  , libGAP_ExecProccallOpts);
 
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_0ARGS ] = libGAP_EvalFunccall0args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_1ARGS ] = libGAP_EvalFunccall1args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_2ARGS ] = libGAP_EvalFunccall2args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_3ARGS ] = libGAP_EvalFunccall3args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_4ARGS ] = libGAP_EvalFunccall4args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_5ARGS ] = libGAP_EvalFunccall5args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_6ARGS ] = libGAP_EvalFunccall6args;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_XARGS ] = libGAP_EvalFunccallXargs;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNCCALL_OPTS  ] = libGAP_EvalFunccallOpts;
-    libGAP_EvalExprFuncs [ libGAP_T_FUNC_EXPR      ] = libGAP_EvalFuncExpr;
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_0ARGS , libGAP_EvalFunccall0args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_1ARGS , libGAP_EvalFunccall1args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_2ARGS , libGAP_EvalFunccall2args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_3ARGS , libGAP_EvalFunccall3args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_4ARGS , libGAP_EvalFunccall4args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_5ARGS , libGAP_EvalFunccall5args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_6ARGS , libGAP_EvalFunccall6args);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_XARGS , libGAP_EvalFunccallXargs);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNCCALL_OPTS  , libGAP_EvalFunccallOpts);
+    libGAP_InstallEvalExprFunc( libGAP_T_FUNC_EXPR      , libGAP_EvalFuncExpr);
 
     /* install the printers                                                */
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_0ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_1ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_2ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_3ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_4ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_5ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_6ARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_XARGS ] = libGAP_PrintProccall;
-    libGAP_PrintStatFuncs[ libGAP_T_PROCCALL_OPTS  ] = libGAP_PrintProccallOpts;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_0ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_1ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_2ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_3ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_4ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_5ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_6ARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_XARGS ] = libGAP_PrintFunccall;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNCCALL_OPTS  ] = libGAP_PrintFunccallOpts;
-    libGAP_PrintExprFuncs[ libGAP_T_FUNC_EXPR      ] = libGAP_PrintFuncExpr;
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_0ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_1ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_2ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_3ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_4ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_5ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_6ARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_XARGS , libGAP_PrintProccall);
+    libGAP_InstallPrintStatFunc( libGAP_T_PROCCALL_OPTS  , libGAP_PrintProccallOpts);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_0ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_1ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_2ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_3ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_4ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_5ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_6ARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_XARGS , libGAP_PrintFunccall);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNCCALL_OPTS  , libGAP_PrintFunccallOpts);
+    libGAP_InstallPrintExprFunc( libGAP_T_FUNC_EXPR      , libGAP_PrintFuncExpr);
 
     /* return success                                                      */
     return 0;
@@ -1509,7 +1595,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoFuncs ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
@@ -1519,6 +1604,3 @@ libGAP_StructInitInfo * libGAP_InitInfoFuncs ( void )
 
 *E  funcs.c . . . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
 */
-
-
-
