@@ -21,6 +21,8 @@
 **
 */
 
+#define _GNU_SOURCE  /* is used for getpt(), ptsname_r prototype etc. */
+
 #include        "system.h"              /* system dependent part           */
 
 #include        "iostream.h"            /* file input/output               */
@@ -42,6 +44,10 @@
 #include        "records.h"             /* generic records                 */
 #include        "bool.h"                /* True and False                  */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 #include        "libgap_internal.h"     /* GAP shared library              */
 
 #include <stdio.h>                      /* standard input/output functions */
@@ -133,18 +139,22 @@ static libGAP_Int libGAP_FreePtyIOStreams;
 libGAP_Int libGAP_NewStream( void )
 {
   libGAP_Int stream = -1;
+  libGAP_HashLock(libGAP_PtyIOStreams);
   if ( libGAP_FreePtyIOStreams != -1 )
   {
       stream = libGAP_FreePtyIOStreams;
       libGAP_FreePtyIOStreams = libGAP_PtyIOStreams[stream].childPID;
   }
+  libGAP_HashUnlock(libGAP_PtyIOStreams);
   return stream;
 }
 
 void libGAP_FreeStream( libGAP_UInt stream)
 {
+   libGAP_HashLock(libGAP_PtyIOStreams);
    libGAP_PtyIOStreams[stream].childPID = libGAP_FreePtyIOStreams;
    libGAP_FreePtyIOStreams = stream;
+   libGAP_HashUnlock(libGAP_PtyIOStreams);
 }
 
 /****************************************************************************
@@ -153,10 +163,12 @@ void libGAP_FreeStream( libGAP_UInt stream)
 */
 void libGAP_SignalChild (libGAP_UInt stream, libGAP_UInt sig)
 {
+    libGAP_HashLock(libGAP_PtyIOStreams);
     if ( libGAP_PtyIOStreams[stream].childPID != -1 )
     {
         kill( libGAP_PtyIOStreams[stream].childPID, sig );
     }
+    libGAP_HashUnlock(libGAP_PtyIOStreams);
 }
 
 /****************************************************************************
@@ -165,11 +177,13 @@ void libGAP_SignalChild (libGAP_UInt stream, libGAP_UInt sig)
 */
 void libGAP_KillChild (libGAP_UInt stream)
 {
+    libGAP_HashLock(libGAP_PtyIOStreams);
     if ( libGAP_PtyIOStreams[stream].childPID != -1 )
     {
         close(libGAP_PtyIOStreams[stream].ptyFD);
         libGAP_SignalChild( stream, SIGKILL );
     }
+    libGAP_HashUnlock(libGAP_PtyIOStreams);
 }
 
 
@@ -255,7 +269,8 @@ static libGAP_UInt libGAP_GetMasterPty ( int *pty, libGAP_Char *nametty, libGAP_
 #elif HAVE_GETPT && HAVE_PTSNAME_R
     /* Attempt to use glibc specific APIs, for compatibility with older
        glibc versions (before 2.2.1, release January 2001). */
-    if ((*pty = getpt()) > 0 ) {
+    *pty = getpt();
+    if (*pty > 0) {
         if (grantpt(*pty) || unlockpt(*pty)) {
             close(*pty);
             return 1;
@@ -270,14 +285,16 @@ static libGAP_UInt libGAP_GetMasterPty ( int *pty, libGAP_Char *nametty, libGAP_
        getpt or openpty, but do have ptsname.
        Platforms *missing* ptsname include:
        Mac OS X 10.3, OpenBSD 3.8, Minix 3.1.8, mingw, MSVC 9, BeOS. */
-    if ( (*pty = open( "/dev/ptmx", O_RDWR )) < 0 )
+    *pty = open( "/dev/ptmx", O_RDWR );
+    if ( *pty < 0 )
         return 1;
     libGAP_strxcpy(nametty, ptsname(*pty), 32);
     return 0;
 
 #elif HAVE_GETPSEUDOTTY
     /* TODO: From which system(s) does getpseudotty originate? */
-    return (*pty = getpseudotty( nametty, namepty )) >= 0 ? 0 : 1;
+    *pty = getpseudotty( nametty, namepty );
+    return *pty < 0 ? 1 : 0;
 
 #elif HAVE__GETPTY
     /* Available on SGI IRIX >= 4.0 (released September 1991).
@@ -289,28 +306,7 @@ static libGAP_UInt libGAP_GetMasterPty ( int *pty, libGAP_Char *nametty, libGAP_
     strcpy( nametty, line );
     return 0;
 
-#elif defined(sgi) || (defined(umips) && defined(USG))
-    /* FIXME: Is this code still usable? A bit later it access an
-       undefined variable 'tty', so for non-sgi systems this won't
-       even compile.
-       */
-    struct stat fstat_buf;
-
-    /* TODO: AIX has /dev/ptc, what else? */
-    *pty = open( "/dev/ptc", O_RDWR );
-    if ( *pty < 0 || (fstat (*pty, &fstat_buf)) < 0 )
-        return 1;
-    snprintf( nametty, 32, "/dev/ttyq%d", minor(fstat_buf.st_rdev) );
-  #if !defined(sgi)
-    snprintf( namepty, 32, "/dev/ptyq%d", minor(fstat_buf.st_rdev) );
-    if ( (*tty = open (nametty, O_RDWR)) < 0 ) {
-        close (*pty);
-        return 1;
-    }
-  #endif
-    return 0;
-
-# else
+#else
     /* fallback to old-style BSD pseudoterminals, doing a brute force
        search over all pty device files. */
     int devindex;
@@ -388,6 +384,7 @@ void libGAP_ChildStatusChanged( int whichsig )
   int status;
   int retcode;
   assert(whichsig == SIGCHLD);
+  libGAP_HashLock(libGAP_PtyIOStreams);
   for (i = 0; i < libGAP_MAX_PTYS; i++) {
       if (libGAP_PtyIOStreams[i].inuse) {
           retcode = waitpid( libGAP_PtyIOStreams[i].childPID, &status, WNOHANG | WUNTRACED );
@@ -398,6 +395,7 @@ void libGAP_ChildStatusChanged( int whichsig )
           }
       }
   }
+  libGAP_HashUnlock(libGAP_PtyIOStreams);
   /* Collect up any other zombie children */
   do {
       retcode = waitpid( -1, &status, WNOHANG);
@@ -558,24 +556,33 @@ void libGAP_HandleChildStatusChanges( libGAP_UInt pty)
 {
   /* common error handling, when we are asked to read or write to a stopped
      or dead child */
+  libGAP_HashLock(libGAP_PtyIOStreams);
   if (libGAP_PtyIOStreams[pty].alive == 0)
   {
       libGAP_PtyIOStreams[pty].changed = 0;
       libGAP_PtyIOStreams[pty].blocked = 0;
+      libGAP_HashUnlock(libGAP_PtyIOStreams);
       libGAP_ErrorQuit("Child Process is unexpectedly dead", (libGAP_Int) 0L, (libGAP_Int) 0L);
+      return;
   }
   if (libGAP_PtyIOStreams[pty].blocked)
   {
+      libGAP_HashUnlock(libGAP_PtyIOStreams);
       libGAP_ErrorQuit("Child Process is still dead", (libGAP_Int)0L,(libGAP_Int)0L);
+      return;
   }
   if (libGAP_PtyIOStreams[pty].changed)
   {
       libGAP_PtyIOStreams[pty].blocked = 1;
       libGAP_PtyIOStreams[pty].changed = 0;
+      libGAP_Int cPID = libGAP_PtyIOStreams[pty].childPID;
+      libGAP_Int status = libGAP_PtyIOStreams[pty].status;
+      libGAP_HashUnlock(libGAP_PtyIOStreams);
       libGAP_ErrorQuit("Child Process %d has stopped or died, status %d",
-                (libGAP_Int) libGAP_PtyIOStreams[pty].childPID,
-                (libGAP_Int) libGAP_PtyIOStreams[pty].status);
+                cPID, status);
+      return;
   }
+  libGAP_HashUnlock(libGAP_PtyIOStreams);
 }
 
 #define libGAP_MAX_ARGS 1000
@@ -922,7 +929,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoIOStream ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
