@@ -148,7 +148,7 @@ libGAP_Obj libGAP_FuncADD_LIST3 (
   libGAP_Int ipos;
   if (pos == (libGAP_Obj)0)
     ipos = -1;
-  else if (libGAP_IS_INTOBJ(pos))
+  else if (libGAP_IS_INTOBJ(pos) && libGAP_INT_INTOBJ(pos) > 0)
     ipos = libGAP_INT_INTOBJ(pos);
   else {
     libGAP_DoOperation3Args( self, list,  obj, pos);
@@ -1676,11 +1676,10 @@ static inline libGAP_Int libGAP_GetIntObj( libGAP_Obj list, libGAP_UInt pos)
       libGAP_Pr("panic: internal inconsistency", 0L, 0L);
       libGAP_SyExit(1);
     }
-  while (!libGAP_IS_INTOBJ(entry))
+  if (!libGAP_IS_INTOBJ(entry))
     {
-      entry = libGAP_ErrorReturnObj("COPY_LIST_ENTRIES: argument %d  must be a small integer, not a %s",
-                             (libGAP_Int)pos, (libGAP_Int)libGAP_InfoBags[libGAP_TNUM_OBJ(entry)].name,
-                             "you can return a small integer to continue");
+      libGAP_ErrorMayQuit("COPY_LIST_ENTRIES: argument %d  must be a small integer, not a %s",
+                   (libGAP_Int)pos, (libGAP_Int)libGAP_InfoBags[libGAP_TNUM_OBJ(entry)].name);
     }
   return libGAP_INT_INTOBJ(entry);
 }
@@ -1688,10 +1687,10 @@ static inline libGAP_Int libGAP_GetIntObj( libGAP_Obj list, libGAP_UInt pos)
 libGAP_Obj libGAP_FuncCOPY_LIST_ENTRIES( libGAP_Obj self, libGAP_Obj args )
 {  
   libGAP_Obj srclist;
-  libGAP_UInt srcstart;
+  libGAP_Int srcstart;
   libGAP_Int srcinc;
   libGAP_Obj dstlist;
-  libGAP_UInt dststart;
+  libGAP_Int dststart;
   libGAP_Int dstinc;
   libGAP_UInt number;
   libGAP_UInt srcmax;
@@ -1715,14 +1714,13 @@ libGAP_Obj libGAP_FuncCOPY_LIST_ENTRIES( libGAP_Obj self, libGAP_Obj args )
       libGAP_Pr("panic: internal inconsistency", 0L, 0L);
       libGAP_SyExit(1);
     }
-  while (!libGAP_IS_PLIST(srclist))
+  if (!libGAP_IS_PLIST(srclist))
     {
-      srclist = libGAP_ErrorReturnObj("COPY_LIST_ENTRIES: source must be a plain list not a %s",
-                               (libGAP_Int)libGAP_InfoBags[libGAP_TNUM_OBJ(srclist)].name, 0L,
-                               "you can return a plain list to continue");
+      libGAP_ErrorMayQuit("COPY_LIST_ENTRIES: source must be a plain list not a %s",
+                   (libGAP_Int)libGAP_InfoBags[libGAP_TNUM_OBJ(srclist)].name, 0L);
     }
 
-  srcstart = (libGAP_UInt)libGAP_GetIntObj(args,2);
+  srcstart = libGAP_GetIntObj(args,2);
   srcinc = libGAP_GetIntObj(args,3);
   dstlist = libGAP_ELM_PLIST(args,4);
   if (!dstlist)
@@ -1732,17 +1730,22 @@ libGAP_Obj libGAP_FuncCOPY_LIST_ENTRIES( libGAP_Obj self, libGAP_Obj args )
     }
   while (!libGAP_IS_PLIST(dstlist) || !libGAP_IS_MUTABLE_OBJ(dstlist))
     {
-      dstlist = libGAP_ErrorReturnObj("COPY_LIST_ENTRIES: destination must be a mutable plain list not a %s",
-                               (libGAP_Int)libGAP_InfoBags[libGAP_TNUM_OBJ(dstlist)].name, 0L,
-                               "you can return a plain list to continue");
+      libGAP_ErrorMayQuit("COPY_LIST_ENTRIES: destination must be a mutable plain list not a %s",
+                   (libGAP_Int)libGAP_InfoBags[libGAP_TNUM_OBJ(dstlist)].name, 0L);
     }
-  dststart = (libGAP_UInt)libGAP_GetIntObj(args,5);
+  dststart = libGAP_GetIntObj(args,5);
   dstinc = libGAP_GetIntObj(args,6);
   number = libGAP_GetIntObj(args,7);
   
   if (number == 0)
     return (libGAP_Obj) 0;
   
+  if ( srcstart <= 0 || dststart <= 0 ||
+       srcstart + (number-1)*srcinc <= 0 || dststart + (number-1)*dstinc <= 0)
+    {
+      libGAP_ErrorMayQuit("COPY_LIST_ENTRIES: list indices must be positive integers",0L,0L);
+    }
+
   srcmax = (srcinc > 0) ? srcstart + (number-1)*srcinc : srcstart;
   dstmax = (dstinc > 0) ? dststart + (number-1)*dstinc : dststart;
   
