@@ -31,6 +31,10 @@
 #include        "string.h"
 #include        <assert.h>
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 /* the following two declarations would belong in `saveload.h', but then all
  * files get macfloat dependencies */
 extern libGAP_Double libGAP_LoadDouble( void);
@@ -50,9 +54,9 @@ extern void libGAP_SaveDouble( libGAP_Double d);
 
 /****************************************************************************
 **
-*F  TypeMacfloat( <macfloat> )  . . . . . . . . . . . . . . . kind of a macfloat value
+*F  TypeMacfloat( <macfloat> )  . . . . . . . . . .  type of a macfloat value
 **
-**  'TypeMacfloat' returns the kind of macfloatean values.
+**  'TypeMacfloat' returns the type of macfloatean values.
 **
 **  'TypeMacfloat' is the function in 'TypeObjFuncs' for macfloatean values.
 */
@@ -362,9 +366,9 @@ libGAP_Obj libGAP_SumIntMacfloat( libGAP_Obj i, libGAP_Obj f )
   }
 
 #define libGAP_MAKEMATHPRIMITIVE2(NAME,name)					\
-  libGAP_Obj Func##NAME##_MACFLOAT( libGAP_Obj self, libGAP_Obj f, libGAP_Obj libGAP_g)			\
+  libGAP_Obj Func##NAME##_MACFLOAT( libGAP_Obj self, libGAP_Obj f, libGAP_Obj g)			\
   {									\
-    return libGAP_NEW_MACFLOAT(libGAP_MATH(name)(libGAP_VAL_MACFLOAT(f),libGAP_VAL_MACFLOAT(libGAP_g)));	\
+    return libGAP_NEW_MACFLOAT(libGAP_MATH(name)(libGAP_VAL_MACFLOAT(f),libGAP_VAL_MACFLOAT(g)));	\
   }
 
 libGAP_MAKEMATHPRIMITIVE(COS,cos)
@@ -570,12 +574,13 @@ static libGAP_Int libGAP_InitKernel (
     /* install the marking functions for macfloatean values                    */
     libGAP_InfoBags[ libGAP_T_MACFLOAT ].name = "macfloat";
     libGAP_InitMarkFuncBags( libGAP_T_MACFLOAT, libGAP_MarkNoSubBags );
+    libGAP_MakeBagTypePublic( libGAP_T_MACFLOAT );
 
     /* init filters and functions                                          */
     libGAP_InitHdlrFiltsFromTable( libGAP_GVarFilts );
     libGAP_InitHdlrFuncsFromTable( libGAP_GVarFuncs );
 
-    /* install the kind function                                           */
+    /* install the type function                                           */
     libGAP_ImportGVarFromLibrary( "TYPE_MACFLOAT", &libGAP_TYPE_MACFLOAT );
     libGAP_TypeObjFuncs[ libGAP_T_MACFLOAT ] = libGAP_TypeMacfloat;
 
@@ -669,7 +674,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoMacfloat ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
