@@ -30,25 +30,29 @@
 
 #include        "objcftl.h"             /* from the left collect           */
 
+#include	"code.h"
+#include	"thread.h"
+#include	"tls.h"
+
 
 #define libGAP_IS_INT_ZERO( n )  (libGAP_IS_INTOBJ(n) && ((n) == libGAP_INTOBJ_INT(0))) 
 
-#define libGAP_GET_COMMUTE( libGAP_g )  libGAP_INT_INTOBJ(libGAP_ELM_PLIST(commute,(libGAP_g))) 
+#define libGAP_GET_COMMUTE( g )  libGAP_INT_INTOBJ(libGAP_ELM_PLIST(commute,(g))) 
 
-#define libGAP_GET_EXPONENT( libGAP_g ) ( ((libGAP_g) <= libGAP_LEN_PLIST(exp)) ? \
-                            libGAP_ELM_PLIST( exp, (libGAP_g) ) : (libGAP_Obj)0 )
-#define libGAP_GET_POWER( libGAP_g )    ( ((libGAP_g) <= libGAP_LEN_PLIST(pow)) ? \
-                            libGAP_ELM_PLIST( pow, (libGAP_g) ) : (libGAP_Obj)0 )
-#define libGAP_GET_IPOWER( libGAP_g )   ( ((libGAP_g) <= libGAP_LEN_PLIST(ipow)) ? \
-                            libGAP_ELM_PLIST( ipow, (libGAP_g) ) : (libGAP_Obj)0 )
+#define libGAP_GET_EXPONENT( g ) ( ((g) <= libGAP_LEN_PLIST(exp)) ? \
+                            libGAP_ELM_PLIST( exp, (g) ) : (libGAP_Obj)0 )
+#define libGAP_GET_POWER( g )    ( ((g) <= libGAP_LEN_PLIST(pow)) ? \
+                            libGAP_ELM_PLIST( pow, (g) ) : (libGAP_Obj)0 )
+#define libGAP_GET_IPOWER( g )   ( ((g) <= libGAP_LEN_PLIST(ipow)) ? \
+                            libGAP_ELM_PLIST( ipow, (g) ) : (libGAP_Obj)0 )
 
-#define libGAP_GET_CONJ( h, libGAP_g ) ( (h <= libGAP_LEN_PLIST( conj ) && \
-                            libGAP_g <= libGAP_LEN_PLIST(libGAP_ELM_PLIST( conj, h ))) ? \
-                           libGAP_ELM_PLIST( libGAP_ELM_PLIST( conj, h ), libGAP_g ) : (libGAP_Obj)0 )
+#define libGAP_GET_CONJ( h, g ) ( (h <= libGAP_LEN_PLIST( conj ) && \
+                            g <= libGAP_LEN_PLIST(libGAP_ELM_PLIST( conj, h ))) ? \
+                           libGAP_ELM_PLIST( libGAP_ELM_PLIST( conj, h ), g ) : (libGAP_Obj)0 )
 
-#define libGAP_GET_ICONJ( h, libGAP_g ) ( (h <= libGAP_LEN_PLIST( iconj ) && \
-                             libGAP_g <= libGAP_LEN_PLIST(libGAP_ELM_PLIST( iconj, h ))) ? \
-                            libGAP_ELM_PLIST( libGAP_ELM_PLIST( iconj, h ), libGAP_g ) : (libGAP_Obj)0 )
+#define libGAP_GET_ICONJ( h, g ) ( (h <= libGAP_LEN_PLIST( iconj ) && \
+                             g <= libGAP_LEN_PLIST(libGAP_ELM_PLIST( iconj, h ))) ? \
+                            libGAP_ELM_PLIST( libGAP_ELM_PLIST( iconj, h ), g ) : (libGAP_Obj)0 )
 
 #define libGAP_PUSH_STACK( word, exp ) {  \
   st++; \
@@ -61,19 +65,19 @@
                                 
 void libGAP_AddIn( libGAP_Obj list, libGAP_Obj w, libGAP_Obj e ) {
 
-  libGAP_Int    libGAP_g,  i;
+  libGAP_Int    g,  i;
   libGAP_Obj    r,  s,  t;
 
   for( i = 1; i < libGAP_LEN_PLIST(w); i += 2 ) {
-      libGAP_g = libGAP_INT_INTOBJ( libGAP_ELM_PLIST( w, i ) );
+      g = libGAP_INT_INTOBJ( libGAP_ELM_PLIST( w, i ) );
 
       s = libGAP_ELM_PLIST( w, i+1 );
       libGAP_C_PROD_FIA( t, s, e );      /*   t = s * e   */
 
-      r = libGAP_ELM_PLIST( list, libGAP_g );
+      r = libGAP_ELM_PLIST( list, g );
       libGAP_C_SUM_FIA( s, t, r );       /*   s = r + s * e   */
 
-      libGAP_SET_ELM_PLIST( list, libGAP_g, s );  libGAP_CHANGED_BAG( list );
+      libGAP_SET_ELM_PLIST( list, g, s );  libGAP_CHANGED_BAG( list );
   }
 
 }
@@ -102,7 +106,7 @@ libGAP_Obj libGAP_CollectPolycyc (
 
     libGAP_Int    st, bottom = libGAP_INT_INTOBJ( libGAP_ADDR_OBJ(pcp)[ libGAP_PC_STACK_POINTER ] );
 
-    libGAP_Int    libGAP_g, syl, h, hh;
+    libGAP_Int    g, syl, h, hh;
 
     libGAP_Obj    e, ee, ge, mge, we, s, t;
     libGAP_Obj    w, x = (libGAP_Obj)0, y = (libGAP_Obj)0;
@@ -126,9 +130,9 @@ libGAP_Obj libGAP_CollectPolycyc (
 
       w   = libGAP_ELM_PLIST( wst, st );
       syl = libGAP_INT_INTOBJ( libGAP_ELM_PLIST( sst, st ) );
-      libGAP_g   = libGAP_INT_INTOBJ( libGAP_ELM_PLIST( w, syl )  );
+      g   = libGAP_INT_INTOBJ( libGAP_ELM_PLIST( w, syl )  );
 
-      if( st > bottom+1 && syl==1 && libGAP_g == libGAP_GET_COMMUTE(libGAP_g) ) {
+      if( st > bottom+1 && syl==1 && g == libGAP_GET_COMMUTE(g) ) {
         /* Collect word^exponent in one go. */
 
         e = libGAP_ELM_PLIST( west, st );
@@ -137,7 +141,7 @@ libGAP_Obj libGAP_CollectPolycyc (
         libGAP_AddIn( list, w, e );
 
         /* Reduce. */
-        for( h = libGAP_g; h <= ngens; h++ ) {
+        for( h = g; h <= ngens; h++ ) {
           s = libGAP_ELM_PLIST( list, h );
           if( libGAP_IS_INT_ZERO( s ) ) continue;
 
@@ -167,8 +171,8 @@ libGAP_Obj libGAP_CollectPolycyc (
 
       }
       else {
-        if( libGAP_g == libGAP_GET_COMMUTE( libGAP_g ) ) {
-          s = libGAP_ELM_PLIST( list, libGAP_g ); 
+        if( g == libGAP_GET_COMMUTE( g ) ) {
+          s = libGAP_ELM_PLIST( list, g ); 
           t = libGAP_ELM_PLIST( est, st ); 
           libGAP_C_SUM_FIA( ge, s, t );
           libGAP_SET_ELM_PLIST( est, st, libGAP_INTOBJ_INT(0) );
@@ -183,7 +187,7 @@ libGAP_Obj libGAP_CollectPolycyc (
             conj  = libGAP_ADDR_OBJ(pcp)[libGAP_PC_CONJUGATES];
             iconj = libGAP_ADDR_OBJ(pcp)[libGAP_PC_INVERSECONJUGATES];
             
-            libGAP_C_SUM_FIA( ge, libGAP_ELM_PLIST( list, libGAP_g ), libGAP_INTOBJ_INT(1) );
+            libGAP_C_SUM_FIA( ge, libGAP_ELM_PLIST( list, g ), libGAP_INTOBJ_INT(1) );
           }
           else {
             libGAP_C_SUM_FIA( ee, e, libGAP_INTOBJ_INT(1) );  e = ee;
@@ -191,28 +195,28 @@ libGAP_Obj libGAP_CollectPolycyc (
             conj  = libGAP_ADDR_OBJ(pcp)[libGAP_PC_CONJUGATESINVERSE];
             iconj = libGAP_ADDR_OBJ(pcp)[libGAP_PC_INVERSECONJUGATESINVERSE];
             
-            libGAP_C_DIFF_FIA( ge, libGAP_ELM_PLIST( list, libGAP_g ), libGAP_INTOBJ_INT(1) );
+            libGAP_C_DIFF_FIA( ge, libGAP_ELM_PLIST( list, g ), libGAP_INTOBJ_INT(1) );
           }
         }
-        libGAP_SET_ELM_PLIST( list, libGAP_g, ge );  libGAP_CHANGED_BAG( list );
+        libGAP_SET_ELM_PLIST( list, g, ge );  libGAP_CHANGED_BAG( list );
 
 
         /* Reduce the exponent.  We delay putting the power onto the 
            stack until all the conjugates are on the stack.  The power is
            stored in  y, its exponent in ge.  */
         y = (libGAP_Obj)0;
-        if( (e = libGAP_GET_EXPONENT( libGAP_g )) ) {
+        if( (e = libGAP_GET_EXPONENT( g )) ) {
             if( !libGAP_LtInt( ge, e ) ) {
                 mge = libGAP_ModInt( ge, e );
-                libGAP_SET_ELM_PLIST( list, libGAP_g, mge ); libGAP_CHANGED_BAG( list );
+                libGAP_SET_ELM_PLIST( list, g, mge ); libGAP_CHANGED_BAG( list );
             
-                if( (y = libGAP_GET_POWER( libGAP_g )) ) ge = libGAP_QuoInt( ge, e );
+                if( (y = libGAP_GET_POWER( g )) ) ge = libGAP_QuoInt( ge, e );
             }
             else if( libGAP_LtInt( ge, libGAP_INTOBJ_INT(0) ) ) {
                 mge = libGAP_ModInt( ge, e );
-                libGAP_SET_ELM_PLIST( list, libGAP_g, mge ); libGAP_CHANGED_BAG( list );
+                libGAP_SET_ELM_PLIST( list, g, mge ); libGAP_CHANGED_BAG( list );
             
-                if( (y = libGAP_GET_IPOWER( libGAP_g )) ) {
+                if( (y = libGAP_GET_IPOWER( g )) ) {
                     ge = libGAP_QuoInt( ge, e );
                     if( !libGAP_IS_INT_ZERO( mge ) ) 
                         ge = libGAP_DiffInt( ge, libGAP_INTOBJ_INT(1) );
@@ -221,24 +225,24 @@ libGAP_Obj libGAP_CollectPolycyc (
             }
         }
         
-        hh = h = libGAP_GET_COMMUTE( libGAP_g );
+        hh = h = libGAP_GET_COMMUTE( g );
         
         /* Find the place where we start to collect. */
-        for( ; h > libGAP_g; h-- ) {
+        for( ; h > g; h-- ) {
             e = libGAP_ELM_PLIST( list, h );
             if( !libGAP_IS_INT_ZERO(e) ) {
             
                 if( libGAP_LtInt( libGAP_INTOBJ_INT(0), e ) ) {
-                    if( libGAP_GET_CONJ( h, libGAP_g ) ) break;
+                    if( libGAP_GET_CONJ( h, g ) ) break;
                 }
                 else {
-                    if( libGAP_GET_ICONJ( h, libGAP_g ) ) break;
+                    if( libGAP_GET_ICONJ( h, g ) ) break;
                 }
             }
         }
 
         /* Put those onto the stack, if necessary. */
-        if( h > libGAP_g || y != (libGAP_Obj)0 ) 
+        if( h > g || y != (libGAP_Obj)0 ) 
           for( ; hh > h; hh-- ) {
             e = libGAP_ELM_PLIST( list, hh );
             if( !libGAP_IS_INT_ZERO(e) ) {
@@ -257,14 +261,14 @@ libGAP_Obj libGAP_CollectPolycyc (
           }
         
         
-        for( ; h > libGAP_g; h-- ) {
+        for( ; h > g; h-- ) {
           e = libGAP_ELM_PLIST( list, h );
           if( !libGAP_IS_INT_ZERO(e) ) {
             libGAP_SET_ELM_PLIST( list, h, libGAP_INTOBJ_INT(0) );
             
             x = (libGAP_Obj)0;
-            if( libGAP_LtInt( libGAP_INTOBJ_INT(0), e ) ) x = libGAP_GET_CONJ( h, libGAP_g );
-            else                            x = libGAP_GET_ICONJ( h, libGAP_g );
+            if( libGAP_LtInt( libGAP_INTOBJ_INT(0), e ) ) x = libGAP_GET_CONJ( h, g );
+            else                            x = libGAP_GET_ICONJ( h, g );
             
             if( x == (libGAP_Obj)0 )  {
               if( libGAP_LtInt( libGAP_INTOBJ_INT(0), e ) ) x = libGAP_ELM_PLIST(  gens, h );
@@ -446,7 +450,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoPcc ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
