@@ -9,6 +9,8 @@
 **
 **  This file contains the functions of the objects package.
 */
+#include	<stdlib.h>
+
 #include        "system.h"              /* Ints, UInts, SyIsIntr           */
 
 
@@ -38,17 +40,50 @@
 
 #include        "saveload.h"            /* saving and loading              */
 
+#include        "aobjects.h"            /* atomic objects                  */
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+//#include	"traverse.h"		/* object traversal		   */
+#include	"tls.h"			/* thread-local storage		   */
+
+
+static libGAP_Int libGAP_lastFreePackageTNUM = libGAP_FIRST_PACKAGE_TNUM;
+
+/****************************************************************************
+**
+*F  RegisterPackageTNUM( <name>, <typeObjFunc> )
+**
+**  Allocates a TNUM for use by a package. The parameters <name> and
+**  <typeObjFunc> are used to initialize the relevant entries in the
+**  InfoBags and TypeObjFuncs arrays.
+**
+**  If allocation fails (e.g. because no more TNUMs are available),
+**  a negative value is returned.
+*/
+libGAP_Int libGAP_RegisterPackageTNUM( const char *name, libGAP_Obj (*typeObjFunc)(libGAP_Obj obj) )
+{
+    if (libGAP_lastFreePackageTNUM > libGAP_LAST_PACKAGE_TNUM)
+        return -1;
+
+    libGAP_Int tnum = libGAP_lastFreePackageTNUM++;
+
+    libGAP_InfoBags[tnum].name = name;
+    libGAP_TypeObjFuncs[tnum] = typeObjFunc;
+
+    return tnum;
+}
+
 
 /****************************************************************************
 **
 
-*F  FamilyTypeHandler( <self>, <kind> ) . . . . . . handler for 'FAMILY_TYPE'
+*F  FamilyTypeHandler( <self>, <type> ) . . . . . . handler for 'FAMILY_TYPE'
 */
 libGAP_Obj libGAP_FamilyTypeHandler (
     libGAP_Obj                 self,
-    libGAP_Obj                 kind )
+    libGAP_Obj                 type )
 {
-    return libGAP_FAMILY_TYPE( kind );
+    return libGAP_FAMILY_TYPE( type );
 }
 
 
@@ -67,13 +102,13 @@ libGAP_Obj libGAP_FamilyObjHandler (
 /****************************************************************************
 **
 
-*F  TYPE_OBJ( <obj> ) . . . . . . . . . . . . . . . . . . . kind of an object
+*F  TYPE_OBJ( <obj> ) . . . . . . . . . . . . . . . . . . . type of an object
 **
-**  'TYPE_OBJ' returns the kind of the object <obj>.
+**  'TYPE_OBJ' returns the type of the object <obj>.
 **
 **  'TYPE_OBJ' is defined in the declaration part of this package.
 */
-libGAP_Obj (*libGAP_TypeObjFuncs[ libGAP_LAST_REAL_TNUM+1 ]) ( libGAP_Obj obj );
+libGAP_Obj (*libGAP_TypeObjFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj obj );
 
 libGAP_Obj libGAP_TypeObjError (
     libGAP_Obj                 obj )
@@ -106,7 +141,7 @@ libGAP_Obj libGAP_TypeObjHandler (
 **
 **  'IS_MUTABLE_OBJ' is defined in the declaration part of this package.
 */
-libGAP_Int (*libGAP_IsMutableObjFuncs[ libGAP_LAST_REAL_TNUM+1 ]) ( libGAP_Obj obj );
+libGAP_Int (*libGAP_IsMutableObjFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj obj );
 
 libGAP_Obj libGAP_IsMutableObjFilt;
 
@@ -153,7 +188,7 @@ libGAP_Obj libGAP_IsMutableObjHandler (
 **
 **  'IS_COPYABLE_OBJ' is defined in the declaration part of this package.
 */
-libGAP_Int (*libGAP_IsCopyableObjFuncs[ libGAP_LAST_REAL_TNUM+1 ]) ( libGAP_Obj obj );
+libGAP_Int (*libGAP_IsCopyableObjFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj obj );
 
 libGAP_Obj libGAP_IsCopyableObjFilt;
 
@@ -196,7 +231,7 @@ libGAP_Obj libGAP_IsCopyableObjHandler (
 
 *V  ShallowCopyObjFuncs[<type>] . . . . . . . . . .  shallow copier functions
 */
-libGAP_Obj (*libGAP_ShallowCopyObjFuncs[ libGAP_LAST_REAL_TNUM+1 ]) ( libGAP_Obj obj );
+libGAP_Obj (*libGAP_ShallowCopyObjFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj obj );
 
 libGAP_Obj libGAP_ShallowCopyObjOper;
 
@@ -812,8 +847,8 @@ static inline libGAP_UInt libGAP_IS_MARKED( libGAP_Obj obj )
   libGAP_UInt i;
   if (!libGAP_IS_MARKABLE(obj))
     return 0;
-  for (i = 0; i < libGAP_PrintObjDepth-1; i++)
-    if (libGAP_PrintObjThiss[i] == obj)
+  for (i = 0; i < libGAP_TLS(libGAP_PrintObjDepth)-1; i++)
+    if (libGAP_TLS(libGAP_PrintObjThiss)[i] == obj)
       return 1;
   return 0;
 }
@@ -840,13 +875,13 @@ void            libGAP_PrintObj (
 
     /* check for interrupts                                                */
     if ( libGAP_SyIsIntr() ) {
-        i = libGAP_PrintObjDepth;
+        i = libGAP_TLS(libGAP_PrintObjDepth);
         libGAP_Pr( "%c%c", (libGAP_Int)'\03', (libGAP_Int)'\04' );
         libGAP_ErrorReturnVoid(
             "user interrupt while printing",
             0L, 0L,
             "you can 'return;'" );
-        libGAP_PrintObjDepth = i;
+        libGAP_TLS(libGAP_PrintObjDepth) = i;
     }
 
     /* First check if <obj> is actually the current object being Viewed
@@ -854,29 +889,29 @@ void            libGAP_PrintObj (
 
     lastPV = libGAP_LastPV;
     libGAP_LastPV = 1;
-    fromview = (lastPV == 2) && (obj == libGAP_PrintObjThis);
-    
+    fromview = (lastPV == 2) && (obj == libGAP_TLS(libGAP_PrintObjThis));
+
     /* if <obj> is a subobject, then mark and remember the superobject
        unless ViewObj has done that job already */
     
-    if ( !fromview  && 0 < libGAP_PrintObjDepth ) {
-        if ( libGAP_IS_MARKABLE(libGAP_PrintObjThis) )  libGAP_MARK( libGAP_PrintObjThis );
-        libGAP_PrintObjThiss[libGAP_PrintObjDepth-1]   = libGAP_PrintObjThis;
-        libGAP_PrintObjIndices[libGAP_PrintObjDepth-1] = libGAP_PrintObjIndex;
+    if ( !fromview  && 0 < libGAP_TLS(libGAP_PrintObjDepth) ) {
+        if ( libGAP_IS_MARKABLE(libGAP_TLS(libGAP_PrintObjThis)) )  libGAP_MARK( libGAP_TLS(libGAP_PrintObjThis) );
+        libGAP_TLS(libGAP_PrintObjThiss)[libGAP_TLS(libGAP_PrintObjDepth)-1]   = libGAP_TLS(libGAP_PrintObjThis);
+        libGAP_TLS(libGAP_PrintObjIndices)[libGAP_TLS(libGAP_PrintObjDepth)-1] = libGAP_TLS(libGAP_PrintObjIndex);
     }
 
     /* handle the <obj>                                                    */
     if (!fromview)
       {
-	libGAP_PrintObjDepth += 1;
-	libGAP_PrintObjThis   = obj;
-	libGAP_PrintObjIndex  = 0;
+	libGAP_TLS(libGAP_PrintObjDepth) += 1;
+	libGAP_TLS(libGAP_PrintObjThis)   = obj;
+	libGAP_TLS(libGAP_PrintObjIndex)  = 0;
       }
 
     /* dispatch to the appropriate printing function                       */
-    if ( (! libGAP_IS_MARKED( libGAP_PrintObjThis )) ) {
-      if (libGAP_PrintObjDepth < libGAP_MAXPRINTDEPTH) {
-        (*libGAP_PrintObjFuncs[ libGAP_TNUM_OBJ(libGAP_PrintObjThis) ])( libGAP_PrintObjThis );
+    if ( (! libGAP_IS_MARKED( libGAP_TLS(libGAP_PrintObjThis) )) ) {
+      if (libGAP_TLS(libGAP_PrintObjDepth) < libGAP_MAXPRINTDEPTH) {
+        (*libGAP_PrintObjFuncs[ libGAP_TNUM_OBJ(libGAP_TLS(libGAP_PrintObjThis)) ])( libGAP_TLS(libGAP_PrintObjThis) );
       }
       else {
         /* don't recurse if depth too high */
@@ -887,9 +922,9 @@ void            libGAP_PrintObj (
     /* or print the path                                                   */
     else {
         libGAP_Pr( "~", 0L, 0L );
-        for ( i = 0; libGAP_PrintObjThis != libGAP_PrintObjThiss[i]; i++ ) {
-            (*libGAP_PrintPathFuncs[ libGAP_TNUM_OBJ(libGAP_PrintObjThiss[i])])
-                ( libGAP_PrintObjThiss[i], libGAP_PrintObjIndices[i] );
+        for ( i = 0; libGAP_TLS(libGAP_PrintObjThis) != libGAP_TLS(libGAP_PrintObjThiss)[i]; i++ ) {
+            (*libGAP_PrintPathFuncs[ libGAP_TNUM_OBJ(libGAP_TLS(libGAP_PrintObjThiss)[i])])
+                ( libGAP_TLS(libGAP_PrintObjThiss)[i], libGAP_TLS(libGAP_PrintObjIndices)[i] );
         }
     }
 
@@ -897,13 +932,13 @@ void            libGAP_PrintObj (
     /* done with <obj>                                                     */
     if (!fromview)
       {
-	libGAP_PrintObjDepth -= 1;
+	libGAP_TLS(libGAP_PrintObjDepth) -= 1;
 	
 	/* if <obj> is a subobject, then restore and unmark the superobject    */
-	if ( 0 < libGAP_PrintObjDepth ) {
-	  libGAP_PrintObjThis  = libGAP_PrintObjThiss[libGAP_PrintObjDepth-1];
-	  libGAP_PrintObjIndex = libGAP_PrintObjIndices[libGAP_PrintObjDepth-1];
-	  if ( libGAP_IS_MARKED(libGAP_PrintObjThis) )  libGAP_UNMARK( libGAP_PrintObjThis );
+	if ( 0 < libGAP_TLS(libGAP_PrintObjDepth) ) {
+	  libGAP_TLS(libGAP_PrintObjThis)  = libGAP_TLS(libGAP_PrintObjThiss)[libGAP_TLS(libGAP_PrintObjDepth)-1];
+	  libGAP_TLS(libGAP_PrintObjIndex) = libGAP_TLS(libGAP_PrintObjIndices)[libGAP_TLS(libGAP_PrintObjDepth)-1];
+	  if ( libGAP_IS_MARKED(libGAP_TLS(libGAP_PrintObjThis)) )  libGAP_UNMARK( libGAP_TLS(libGAP_PrintObjThis) );
 	}
       }
     libGAP_LastPV = lastPV;
@@ -953,7 +988,7 @@ libGAP_Obj libGAP_PrintObjHandler (
 libGAP_Obj libGAP_FuncSET_PRINT_OBJ_INDEX (libGAP_Obj self, libGAP_Obj ind)
 {
   if (libGAP_IS_INTOBJ(ind))
-    libGAP_PrintObjIndex = libGAP_INT_INTOBJ(ind);
+    libGAP_TLS(libGAP_PrintObjIndex) = libGAP_INT_INTOBJ(ind);
   return 0;
 }
 
@@ -980,21 +1015,21 @@ void            libGAP_ViewObj (
     libGAP_LastPV = 2;
     
     /* if <obj> is a subobject, then mark and remember the superobject     */
-    if ( 0 < libGAP_PrintObjDepth ) {
-        if ( libGAP_IS_MARKABLE(libGAP_PrintObjThis) )  libGAP_MARK( libGAP_PrintObjThis );
-        libGAP_PrintObjThiss[libGAP_PrintObjDepth-1]   = libGAP_PrintObjThis;
-        libGAP_PrintObjIndices[libGAP_PrintObjDepth-1] =  libGAP_PrintObjIndex;
+    if ( 0 < libGAP_TLS(libGAP_PrintObjDepth) ) {
+        if ( libGAP_IS_MARKABLE(libGAP_TLS(libGAP_PrintObjThis)) )  libGAP_MARK( libGAP_TLS(libGAP_PrintObjThis) );
+        libGAP_TLS(libGAP_PrintObjThiss)[libGAP_TLS(libGAP_PrintObjDepth)-1]   = libGAP_TLS(libGAP_PrintObjThis);
+        libGAP_TLS(libGAP_PrintObjIndices)[libGAP_TLS(libGAP_PrintObjDepth)-1] =  libGAP_TLS(libGAP_PrintObjIndex);
     }
 
     /* handle the <obj>                                                    */
-    libGAP_PrintObjDepth += 1;
-    libGAP_PrintObjThis   = obj;
-    libGAP_PrintObjIndex  = 0;
+    libGAP_TLS(libGAP_PrintObjDepth) += 1;
+    libGAP_TLS(libGAP_PrintObjThis)   = obj;
+    libGAP_TLS(libGAP_PrintObjIndex)  = 0;
 
     /* dispatch to the appropriate viewing function                       */
 
-    if ( ! libGAP_IS_MARKED( libGAP_PrintObjThis ) ) {
-      if (libGAP_PrintObjDepth < libGAP_MAXPRINTDEPTH) {
+    if ( ! libGAP_IS_MARKED( libGAP_TLS(libGAP_PrintObjThis) ) ) {
+      if (libGAP_TLS(libGAP_PrintObjDepth) < libGAP_MAXPRINTDEPTH) {
         libGAP_DoOperation1Args( libGAP_ViewObjOper, obj );
       }
       else {
@@ -1006,20 +1041,20 @@ void            libGAP_ViewObj (
     /* or view the path                                                   */
     else {
         libGAP_Pr( "~", 0L, 0L );
-        for ( i = 0; libGAP_PrintObjThis != libGAP_PrintObjThiss[i]; i++ ) {
-            (*libGAP_PrintPathFuncs[ libGAP_TNUM_OBJ(libGAP_PrintObjThiss[i]) ])
-                ( libGAP_PrintObjThiss[i], libGAP_PrintObjIndices[i] );
+        for ( i = 0; libGAP_TLS(libGAP_PrintObjThis) != libGAP_TLS(libGAP_PrintObjThiss)[i]; i++ ) {
+            (*libGAP_PrintPathFuncs[ libGAP_TNUM_OBJ(libGAP_TLS(libGAP_PrintObjThiss)[i]) ])
+                ( libGAP_TLS(libGAP_PrintObjThiss)[i], libGAP_TLS(libGAP_PrintObjIndices)[i] );
         }
     }
 
     /* done with <obj>                                                     */
-    libGAP_PrintObjDepth -= 1;
+    libGAP_TLS(libGAP_PrintObjDepth) -= 1;
 
     /* if <obj> is a subobject, then restore and unmark the superobject    */
-    if ( 0 < libGAP_PrintObjDepth ) {
-        libGAP_PrintObjThis  = libGAP_PrintObjThiss[libGAP_PrintObjDepth-1];
-        libGAP_PrintObjIndex = libGAP_PrintObjIndices[libGAP_PrintObjDepth-1];
-        if ( libGAP_IS_MARKED(libGAP_PrintObjThis) )  libGAP_UNMARK( libGAP_PrintObjThis );
+    if ( 0 < libGAP_TLS(libGAP_PrintObjDepth) ) {
+        libGAP_TLS(libGAP_PrintObjThis)  = libGAP_TLS(libGAP_PrintObjThiss)[libGAP_TLS(libGAP_PrintObjDepth)-1];
+        libGAP_TLS(libGAP_PrintObjIndex) = libGAP_TLS(libGAP_PrintObjIndices)[libGAP_TLS(libGAP_PrintObjDepth)-1];
+        if ( libGAP_IS_MARKED(libGAP_TLS(libGAP_PrintObjThis)) )  libGAP_UNMARK( libGAP_TLS(libGAP_PrintObjThis) );
     }
 
     libGAP_LastPV = lastPV;
@@ -1087,14 +1122,14 @@ libGAP_Obj             libGAP_IS_COMOBJ_Handler (
 
 /****************************************************************************
 **
-*F  SET_TYPE_COMOBJ_Handler( <self>, <obj>, <kind> ) . . .  'SET_TYPE_COMOBJ'
+*F  SET_TYPE_COMOBJ_Handler( <self>, <obj>, <type> ) . . .  'SET_TYPE_COMOBJ'
 */
 libGAP_Obj libGAP_SET_TYPE_COMOBJ_Handler (
     libGAP_Obj                 self,
     libGAP_Obj                 obj,
-    libGAP_Obj                 kind )
+    libGAP_Obj                 type )
 {
-    libGAP_TYPE_COMOBJ( obj ) = kind;
+    libGAP_TYPE_COMOBJ( obj ) = type;
     libGAP_RetypeBag( obj, libGAP_T_COMOBJ );
     libGAP_CHANGED_BAG( obj );
     return obj;
@@ -1127,14 +1162,14 @@ libGAP_Obj libGAP_IS_POSOBJ_Handler (
 
 /****************************************************************************
 **
-*F  SET_TYPE_POSOBJ_Handler( <self>, <obj>, <kind> )  . . .  'SET_TYPE_POSOB'
+*F  SET_TYPE_POSOBJ_Handler( <self>, <obj>, <type> )  . . .  'SET_TYPE_POSOB'
 */
 libGAP_Obj libGAP_SET_TYPE_POSOBJ_Handler (
     libGAP_Obj                 self,
     libGAP_Obj                 obj,
-    libGAP_Obj                 kind )
+    libGAP_Obj                 type )
 {
-    libGAP_TYPE_POSOBJ( obj ) = kind;
+    libGAP_TYPE_POSOBJ( obj ) = type;
     libGAP_RetypeBag( obj, libGAP_T_POSOBJ );
     libGAP_CHANGED_BAG( obj );
     return obj;
@@ -1179,14 +1214,14 @@ libGAP_Obj             libGAP_IS_DATOBJ_Handler (
 
 /****************************************************************************
 **
-*F  SET_TYPE_DATOBJ_Handler( <self>, <obj>, <kind> ) . . .  'SET_TYPE_DATOBJ'
+*F  SET_TYPE_DATOBJ_Handler( <self>, <obj>, <type> ) . . .  'SET_TYPE_DATOBJ'
 */
 libGAP_Obj libGAP_SET_TYPE_DATOBJ_Handler (
     libGAP_Obj                 self,
     libGAP_Obj                 obj,
-    libGAP_Obj                 kind )
+    libGAP_Obj                 type )
 {
-    libGAP_TYPE_DATOBJ( obj ) = kind;
+    libGAP_TYPE_DATOBJ( obj ) = type;
     libGAP_RetypeBag( obj, libGAP_T_DATOBJ );
     libGAP_CHANGED_BAG( obj );
     return obj;
@@ -1223,11 +1258,9 @@ libGAP_Obj libGAP_IsIdenticalHandler (
 **  No saving function may allocate any bag
 */
 
-void (*libGAP_SaveObjFuncs[ 256 ]) (libGAP_Obj obj);
+void (*libGAP_SaveObjFuncs[256]) ( libGAP_Obj obj );
 
-void libGAP_SaveObjError (
-                   libGAP_Obj obj
-                   )
+void libGAP_SaveObjError( libGAP_Obj obj )
 {
   libGAP_ErrorQuit(
             "Panic: tried to save an object of unknown type '%d'",
@@ -1252,11 +1285,9 @@ void libGAP_SaveObjError (
 **  No loading function may allocate any bag
 */
 
-void (*libGAP_LoadObjFuncs[ 256 ]) (libGAP_Bag bag);
+void (*libGAP_LoadObjFuncs[256]) ( libGAP_Obj obj );
 
-void libGAP_LoadObjError (
-                   libGAP_Obj obj
-                   )
+void libGAP_LoadObjError( libGAP_Obj obj )
 {
   libGAP_ErrorQuit(
             "Panic: tried to load an object of unknown type '%d'",
@@ -1460,6 +1491,52 @@ libGAP_Obj libGAP_FuncCLONE_OBJ (
     return 0;
 }
 
+/****************************************************************************
+**
+
+*F  FuncSWITCH_OBJ( <self>, <obj1>, <obj2> ) . . .  switch <obj1> and <obj2>
+**
+**  `SWITCH_OBJ' exchanges the objects referenced by its two arguments.  It
+**   is not allowed to switch clone small integers or finite field elements.
+**
+**   This is inspired by the Smalltalk 'become:' operation.
+*/
+
+libGAP_Obj libGAP_FuncSWITCH_OBJ(libGAP_Obj self, libGAP_Obj obj1, libGAP_Obj obj2) {
+    libGAP_Obj *ptr1, *ptr2;
+
+    if ( libGAP_IS_INTOBJ(obj1) || libGAP_IS_INTOBJ(obj2) ) {
+        libGAP_ErrorReturnVoid( "small integer objects cannot be switched", 0, 0,
+                         "you can 'return;' to leave them in place" );
+        return 0;
+    }
+    if ( libGAP_IS_FFE(obj1) || libGAP_IS_FFE(obj2) ) {
+        libGAP_ErrorReturnVoid( "finite field elements cannot be switched", 0, 0,
+                         "you can 'return;' to leave them in place" );
+        return 0;
+    }
+    ptr1 = libGAP_PTR_BAG(obj1);
+    ptr2 = libGAP_PTR_BAG(obj2);
+    libGAP_PTR_BAG(obj2) = ptr1;
+    libGAP_PTR_BAG(obj1) = ptr2;
+    libGAP_CHANGED_BAG(obj1);
+    libGAP_CHANGED_BAG(obj2);
+    return (libGAP_Obj) 0;
+}
+
+
+/****************************************************************************
+**
+
+*F  FuncFORCE_SWITCH_OBJ( <self>, <obj1>, <obj2> ) .  switch <obj1> and <obj2>
+**
+**  In GAP, FORCE_SWITCH_OBJ does the same thing as SWITCH_OBJ. In HPC_GAP
+**  it allows public objects to be exchanged.
+*/
+
+libGAP_Obj libGAP_FuncFORCE_SWITCH_OBJ(libGAP_Obj self, libGAP_Obj obj1, libGAP_Obj obj2) {
+    return libGAP_FuncSWITCH_OBJ(self, obj1, obj2);
+}
 
 /****************************************************************************
 **
@@ -1512,7 +1589,7 @@ static libGAP_StructGVarOper libGAP_GVarOpers [] = {
 */
 static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
 
-    { "FAMILY_TYPE", 1, "kind",
+    { "FAMILY_TYPE", 1, "type",
       libGAP_FamilyTypeHandler, "src/objects.c:FAMILY_TYPE" },
 
     { "TYPE_OBJ", 1, "obj",
@@ -1554,6 +1631,12 @@ static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
     { "CLONE_OBJ", 2, "obj, dst, src",
       libGAP_FuncCLONE_OBJ, "src/objects.c:CLONE_OBJ" },
 
+    { "SWITCH_OBJ", 2, "obj1, obj2",
+      libGAP_FuncSWITCH_OBJ, "src/objects.c:SWITCH_OBJ" },
+
+    { "FORCE_SWITCH_OBJ", 2, "obj1, obj2",
+      libGAP_FuncFORCE_SWITCH_OBJ, "src/objects.c:FORCE_SWITCH_OBJ" },
+
     { "SET_PRINT_OBJ_INDEX", 1, "index",
       libGAP_FuncSET_PRINT_OBJ_INDEX, "src/objects.c:SET_PRINT_OBJ_INDEX" },
 
@@ -1589,9 +1672,9 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_InfoBags[         libGAP_T_DATOBJ +libGAP_COPYING ].name = "object (data,copied)";
     libGAP_InitMarkFuncBags( libGAP_T_DATOBJ +libGAP_COPYING , libGAP_MarkOneSubBags  );
 
-
-    for ( t = libGAP_FIRST_REAL_TNUM; t <= libGAP_LAST_REAL_TNUM; t++ )
+    for ( t = libGAP_FIRST_REAL_TNUM; t <= libGAP_LAST_REAL_TNUM; t++ ) {
         libGAP_TypeObjFuncs[ t ] = libGAP_TypeObjError;
+    }
 
     libGAP_TypeObjFuncs[ libGAP_T_COMOBJ ] = libGAP_TypeComObj;
     libGAP_TypeObjFuncs[ libGAP_T_POSOBJ ] = libGAP_TypePosObj;
@@ -1784,7 +1867,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoObjects ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
@@ -1794,4 +1876,3 @@ libGAP_StructInitInfo * libGAP_InitInfoObjects ( void )
 
 *E  objects.c . . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
 */
-
