@@ -47,6 +47,10 @@
 #include        "set.h"                 /* plain sets                      */
 #include        "string.h"              /* strings                         */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 
 /****************************************************************************
 **
@@ -500,7 +504,7 @@ libGAP_Obj libGAP_FuncADD_SET (
 	if (len == 0 )
 	  {
 	    if (libGAP_TNUM_OBJ(obj) <= libGAP_T_CYC)
-	      libGAP_RetypeBag( set, libGAP_T_PLIST_CYC_SSORT);
+	      libGAP_RetypeBagIfWritable( set, libGAP_T_PLIST_CYC_SSORT);
 	    else
 	      {
 		libGAP_SET_FILT_LIST( set, libGAP_FN_IS_HOMOG );
@@ -514,7 +518,7 @@ libGAP_Obj libGAP_FuncADD_SET (
 	    /* Now determine homogeneity */
 	    if (isCyc)
 	      if (libGAP_TNUM_OBJ(obj) <= libGAP_T_CYC)
-		libGAP_RetypeBag( set, libGAP_T_PLIST_CYC_SSORT);
+		libGAP_RetypeBagIfWritable( set, libGAP_T_PLIST_CYC_SSORT);
 	      else
 		{
 		  libGAP_RESET_FILT_LIST(set, libGAP_FN_IS_HOMOG);
@@ -1170,7 +1174,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoSet ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
