@@ -98,13 +98,25 @@ libGAP_Int libGAP_READ_COMMAND ( void )
     return 1;
 }
 
-libGAP_Obj libGAP_FuncREAD_COMMAND ( libGAP_Obj self, libGAP_Obj stream, libGAP_Obj echo )
+/*
+ Returns a list with one or two entries. The first
+ entry is set to "false" if there was any error
+ executing the command, and "true" otherwise.
+ The second entry, if present, is the return value of
+ the command. If it not present, the command returned nothing.
+*/
+libGAP_Obj libGAP_FuncREAD_COMMAND_REAL ( libGAP_Obj self, libGAP_Obj stream, libGAP_Obj echo )
 {
     libGAP_Int status;
+    libGAP_Obj result;
+
+    result = libGAP_NEW_PLIST( libGAP_T_PLIST, 2 );
+    libGAP_SET_LEN_PLIST(result, 1);
+    libGAP_SET_ELM_PLIST(result, 1, libGAP_False);
 
     /* try to open the file                                                */
     if ( ! libGAP_OpenInputStream(stream) ) {
-        return libGAP_SFail;
+        return result;
     }
 
     if (echo == libGAP_True)
@@ -116,19 +128,34 @@ libGAP_Obj libGAP_FuncREAD_COMMAND ( libGAP_Obj self, libGAP_Obj stream, libGAP_
     
     libGAP_CloseInput();
 
-    if( status == 0 ) return libGAP_SFail;
+    if( status == 0 ) return result;
 
     if (libGAP_TLS(libGAP_UserHasQUIT)) {
       libGAP_TLS(libGAP_UserHasQUIT) = 0;
-      return libGAP_SFail;
+      return result;
     }
 
     if (libGAP_TLS(libGAP_UserHasQuit)) {
       libGAP_TLS(libGAP_UserHasQuit) = 0;
     }
     
-    return libGAP_TLS(libGAP_ReadEvalResult) ? libGAP_TLS(libGAP_ReadEvalResult) : libGAP_SFail;
-    
+    libGAP_SET_ELM_PLIST(result, 1, libGAP_True);
+    if (libGAP_TLS(libGAP_ReadEvalResult)) {
+        libGAP_SET_LEN_PLIST(result, 2);
+        libGAP_SET_ELM_PLIST(result, 2, libGAP_TLS(libGAP_ReadEvalResult));
+    }
+    return result;
+}
+
+/*
+ Deprecated alternative to READ_COMMAND_REAL, kept for now to maintain
+ compatibility with the few packages that use it.
+ */
+libGAP_Obj libGAP_FuncREAD_COMMAND ( libGAP_Obj self, libGAP_Obj stream, libGAP_Obj echo )
+{
+    libGAP_Obj result;
+    result = libGAP_FuncREAD_COMMAND_REAL(self, stream, echo);
+    return (libGAP_LEN_PLIST(result) == 2) ? libGAP_ELM_PLIST(result, 2) : libGAP_SuPeRfail;
 }
 
 /****************************************************************************
@@ -1089,6 +1116,8 @@ libGAP_Obj libGAP_FuncREAD_GAP_ROOT (
     libGAP_Obj                 self,
     libGAP_Obj                 filename )
 {
+    libGAP_Char filenamecpy[4096];
+
     /* check the argument                                                  */
     while ( ! libGAP_IsStringConv( filename ) ) {
         filename = libGAP_ErrorReturnObj(
@@ -1097,8 +1126,10 @@ libGAP_Obj libGAP_FuncREAD_GAP_ROOT (
             "you can replace <filename> via 'return <filename>;'" );
     }
 
+    /* Copy to avoid garbage collection moving string                      */
+    libGAP_strlcpy(filenamecpy, libGAP_CSTR_STRING(filename), 4096);
     /* try to open the file                                                */
-    return libGAP_READ_GAP_ROOT(libGAP_CSTR_STRING(filename)) ? libGAP_True : libGAP_False;
+    return libGAP_READ_GAP_ROOT(filenamecpy) ? libGAP_True : libGAP_False;
 }
 
 
@@ -2234,6 +2265,9 @@ static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
     { "READ_NORECOVERY", 1L, "filename",
       libGAP_FuncREAD_NORECOVERY, "src/streams.c:READ_NORECOVERY" },
 
+    { "READ_COMMAND_REAL", 2L, "stream, echo",
+      libGAP_FuncREAD_COMMAND_REAL, "src/streams.c:READ_COMMAND_REAL" },
+
     { "READ_COMMAND", 2L, "stream, echo", 
       libGAP_FuncREAD_COMMAND, "src/streams.c:READ_COMMAND" },
 
